import random

ADJECTIVES = [
    "abandoned",
    "able",
    "absolute",
    "adorable",
    "adventurous",
    "academic",
    "acceptable",
    "acclaimed",
    "accomplished",
    "accurate",
    "aching",
    "acidic",
    "acrobatic",
    "active",
    "actual",
    "adept",
    "admirable",
    "admired",
    "adolescent",
    "adorable",
    "adored",
    "advanced",
    "afraid",
    "affectionate",
    "aged",
    "aggravating",
    "aggressive",
    "agile",
    "agitated",
    "agonizing",
    "agreeable",
    "ajar",
    "alarmed",
    "alarming",
    "alert",
    "alienated",
    "alive",
    "all",
    "altruistic",
    "amazing",
    "ambitious",
    "ample",
    "amused",
    "amusing",
    "anchored",
    "ancient",
    "angelic",
    "angry",
    "anguished",
    "animated",
    "annual",
    "another",
    "antique",
    "anxious",
    "any",
    "apprehensive",
    "appropriate",
    "apt",
    "arctic",
    "arid",
    "aromatic",
    "artistic",
    "ashamed",
    "assured",
    "astonishing",
    "athletic",
    "attached",
    "attentive",
    "attractive",
    "austere",
    "authentic",
    "authorized",
    "automatic",
    "avaricious",
    "average",
    "aware",
    "awesome",
    "awful",
    "awkward",
    "babyish",
    "bad",
    "back",
    "baggy",
    "bare",
    "barren",
    "basic",
    "beautiful",
    "belated",
    "beloved",
    "beneficial",
    "better",
    "best",
    "bewitched",
    "big",
    "big-hearted",
    "biodegradable",
    "bite-sized",
    "bitter",
    "black",
    "black-and-white",
    "bland",
    "blank",
    "blaring",
    "bleak",
    "blind",
    "blissful",
    "blond",
    "blue",
    "blushing",
    "bogus",
    "boiling",
    "bold",
    "bony",
    "boring",
    "bossy",
    "both",
    "bouncy",
    "bountiful",
    "bowed",
    "brave",
    "breakable",
    "brief",
    "bright",
    "brilliant",
    "brisk",
    "broken",
    "bronze",
    "brown",
    "bruised",
    "bubbly",
    "bulky",
    "bumpy",
    "buoyant",
    "burdensome",
    "burly",
    "bustling",
    "busy",
    "buttery",
    "buzzing",
    "calculating",
    "calm",
    "candid",
    "canine",
    "capital",
    "carefree",
    "careful",
    "careless",
    "caring",
    "cautious",
    "cavernous",
    "celebrated",
    "charming",
    "cheap",
    "cheerful",
    "cheery",
    "chief",
    "chilly",
    "chubby",
    "circular",
    "classic",
    "clean",
    "clear",
    "clear-cut",
    "clever",
    "close",
    "closed",
    "cloudy",
    "clueless",
    "clumsy",
    "cluttered",
    "coarse",
    "cold",
    "colorful",
    "colorless",
    "colossal",
    "comfortable",
    "common",
    "compassionate",
    "competent",
    "complete",
    "complex",
    "complicated",
    "composed",
    "concerned",
    "concrete",
    "confused",
    "conscious",
    "considerate",
    "constant",
    "content",
    "conventional",
    "cooked",
    "cool",
    "cooperative",
    "coordinated",
    "corny",
    "corrupt",
    "costly",
    "courageous",
    "courteous",
    "crafty",
    "crazy",
    "creamy",
    "creative",
    "creepy",
    "criminal",
    "crisp",
    "critical",
    "crooked",
    "crowded",
    "cruel",
    "crushing",
    "cuddly",
    "cultivated",
    "cultured",
    "cumbersome",
    "curly",
    "curvy",
    "cute",
    "cylindrical",
    "damaged",
    "damp",
    "dangerous",
    "dapper",
    "daring",
    "darling",
    "dark",
    "dazzling",
    "dead",
    "deadly",
    "deafening",
    "dear",
    "dearest",
    "decent",
    "decimal",
    "decisive",
    "deep",
    "defenseless",
    "defensive",
    "defiant",
    "deficient",
    "definite",
    "definitive",
    "delayed",
    "delectable",
    "delicious",
    "delightful",
    "delirious",
    "demanding",
    "dense",
    "dental",
    "dependable",
    "dependent",
    "descriptive",
    "deserted",
    "detailed",
    "determined",
    "devoted",
    "different",
    "difficult",
    "digital",
    "diligent",
    "dim",
    "dimpled",
    "dimwitted",
    "direct",
    "disastrous",
    "discrete",
    "disfigured",
    "disgusting",
    "disloyal",
    "dismal",
    "distant",
    "downright",
    "dreary",
    "dirty",
    "disguised",
    "dishonest",
    "dismal",
    "distant",
    "distinct",
    "distorted",
    "dizzy",
    "dopey",
    "doting",
    "double",
    "downright",
    "drab",
    "drafty",
    "dramatic",
    "dreary",
    "droopy",
    "dry",
    "dual",
    "dull",
    "dutiful",
    "E",
    "each",
    "eager",
    "earnest",
    "early",
    "easy",
    "easy-going",
    "ecstatic",
    "edible",
    "educated",
    "elaborate",
    "elastic",
    "elated",
    "elderly",
    "electric",
    "elegant",
    "elementary",
    "elliptical",
    "embarrassed",
    "embellished",
    "eminent",
    "emotional",
    "empty",
    "enchanted",
    "enchanting",
    "energetic",
    "enlightened",
    "enormous",
    "enraged",
    "entire",
    "envious",
    "equal",
    "equatorial",
    "essential",
    "esteemed",
    "ethical",
    "euphoric",
    "even",
    "evergreen",
    "everlasting",
    "every",
    "evil",
    "exalted",
    "excellent",
    "exemplary",
    "exhausted",
    "excitable",
    "excited",
    "exciting",
    "exotic",
    "expensive",
    "experienced",
    "expert",
    "extraneous",
    "extroverted",
    "extra-large",
    "extra-small",
    "F",
    "fabulous",
    "failing",
    "faint",
    "fair",
    "faithful",
    "fake",
    "false",
    "familiar",
    "famous",
    "fancy",
    "fantastic",
    "far",
    "faraway",
    "far-flung",
    "far-off",
    "fast",
    "fat",
    "fatal",
    "fatherly",
    "favorable",
    "favorite",
    "fearful",
    "fearless",
    "feisty",
    "feline",
    "female",
    "feminine",
    "few",
    "fickle",
    "filthy",
    "fine",
    "finished",
    "firm",
    "first",
    "firsthand",
    "fitting",
    "fixed",
    "flaky",
    "flamboyant",
    "flashy",
    "flat",
    "flawed",
    "flawless",
    "flickering",
    "flimsy",
    "flippant",
    "flowery",
    "fluffy",
    "fluid",
    "flustered",
    "focused",
    "fond",
    "foolhardy",
    "foolish",
    "forceful",
    "forked",
    "formal",
    "forsaken",
    "forthright",
    "fortunate",
    "fragrant",
    "frail",
    "frank",
    "frayed",
    "free",
    "French",
    "fresh",
    "frequent",
    "friendly",
    "frightened",
    "frightening",
    "frigid",
    "frilly",
    "frizzy",
    "frivolous",
    "front",
    "frosty",
    "frozen",
    "frugal",
    "fruitful",
    "full",
    "fumbling",
    "functional",
    "funny",
    "fussy",
    "fuzzy",
    "gargantuan",
    "gaseous",
    "general",
    "generous",
    "gentle",
    "genuine",
    "giant",
    "giddy",
    "gigantic",
    "gifted",
    "giving",
    "glamorous",
    "glaring",
    "glass",
    "gleaming",
    "gleeful",
    "glistening",
    "glittering",
    "gloomy",
    "glorious",
    "glossy",
    "glum",
    "golden",
    "good",
    "good-natured",
    "gorgeous",
    "graceful",
    "gracious",
    "grand",
    "grandiose",
    "granular",
    "grateful",
    "grave",
    "gray",
    "great",
    "greedy",
    "green",
    "gregarious",
    "grim",
    "grimy",
    "gripping",
    "grizzled",
    "gross",
    "grotesque",
    "grouchy",
    "grounded",
    "growing",
    "growling",
    "grown",
    "grubby",
    "gruesome",
    "grumpy",
    "guilty",
    "gullible",
    "gummy",
    "hairy",
    "half",
    "handmade",
    "handsome",
    "handy",
    "happy",
    "happy-go-lucky",
    "hard",
    "hard-to-find",
    "harmful",
    "harmless",
    "harmonious",
    "harsh",
    "hasty",
    "hateful",
    "haunting",
    "healthy",
    "heartfelt",
    "hearty",
    "heavenly",
    "heavy",
    "hefty",
    "helpful",
    "helpless",
    "hidden",
    "hideous",
    "high",
    "high-level",
    "hilarious",
    "hoarse",
    "hollow",
    "homely",
    "honest",
    "honorable",
    "honored",
    "hopeful",
    "horrible",
    "hospitable",
    "hot",
    "huge",
    "humble",
    "humiliating",
    "humming",
    "humongous",
    "hungry",
    "hurtful",
    "husky",
    "icky",
    "icy",
    "ideal",
    "idealistic",
    "identical",
    "idle",
    "idiotic",
    "idolized",
    "ignorant",
    "ill",
    "illegal",
    "ill-fated",
    "ill-informed",
    "illiterate",
    "illustrious",
    "imaginary",
    "imaginative",
    "immaculate",
    "immaterial",
    "immediate",
    "immense",
    "impassioned",
    "impeccable",
    "impartial",
    "imperfect",
    "imperturbable",
    "impish",
    "impolite",
    "important",
    "impossible",
    "impractical",
    "impressionable",
    "impressive",
    "improbable",
    "impure",
    "inborn",
    "incomparable",
    "incompatible",
    "incomplete",
    "inconsequential",
    "incredible",
    "indelible",
    "inexperienced",
    "indolent",
    "infamous",
    "infantile",
    "infatuated",
    "inferior",
    "infinite",
    "informal",
    "innocent",
    "insecure",
    "insidious",
    "insignificant",
    "insistent",
    "instructive",
    "insubstantial",
    "intelligent",
    "intent",
    "intentional",
    "interesting",
    "internal",
    "international",
    "intrepid",
    "ironclad",
    "irresponsible",
    "irritating",
    "itchy",
    "jaded",
    "jagged",
    "jam-packed",
    "jaunty",
    "jealous",
    "jittery",
    "joint",
    "jolly",
    "jovial",
    "joyful",
    "joyous",
    "jubilant",
    "judicious",
    "juicy",
    "jumbo",
    "junior",
    "jumpy",
    "juvenile",
    "kaleidoscopic",
    "keen",
    "key",
    "kind",
    "kindhearted",
    "kindly",
    "klutzy",
    "knobby",
    "knotty",
    "knowledgeable",
    "knowing",
    "known",
    "kooky",
    "kosher",
    "lame",
    "lanky",
    "large",
    "last",
    "lasting",
    "late",
    "lavish",
    "lawful",
    "lazy",
    "leading",
    "lean",
    "leafy",
    "left",
    "legal",
    "legitimate",
    "light",
    "lighthearted",
    "likable",
    "likely",
    "limited",
    "limp",
    "limping",
    "linear",
    "lined",
    "liquid",
    "little",
    "live",
    "lively",
    "livid",
    "loathsome",
    "lone",
    "lonely",
    "long",
    "long-term",
    "loose",
    "lopsided",
    "lost",
    "loud",
    "lovable",
    "lovely",
    "loving",
    "low",
    "loyal",
    "lucky",
    "lumbering",
    "luminous",
    "lumpy",
    "lustrous",
    "luxurious",
    "mad",
    "made-up",
    "magnificent",
    "majestic",
    "major",
    "male",
    "mammoth",
    "married",
    "marvelous",
    "masculine",
    "massive",
    "mature",
    "meager",
    "mealy",
    "mean",
    "measly",
    "meaty",
    "medical",
    "mediocre",
    "medium",
    "meek",
    "mellow",
    "melodic",
    "memorable",
    "menacing",
    "merry",
    "messy",
    "metallic",
    "mild",
    "milky",
    "mindless",
    "miniature",
    "minor",
    "minty",
    "miserable",
    "miserly",
    "misguided",
    "misty",
    "mixed",
    "modern",
    "modest",
    "moist",
    "monstrous",
    "monthly",
    "monumental",
    "moral",
    "mortified",
    "motherly",
    "motionless",
    "mountainous",
    "muddy",
    "muffled",
    "multicolored",
    "mundane",
    "murky",
    "mushy",
    "musty",
    "muted",
    "mysterious",
    "naive",
    "narrow",
    "nasty",
    "natural",
    "naughty",
    "nautical",
    "near",
    "neat",
    "necessary",
    "needy",
    "negative",
    "neglected",
    "negligible",
    "neighboring",
    "nervous",
    "new",
    "next",
    "nice",
    "nifty",
    "nimble",
    "nippy",
    "nocturnal",
    "noisy",
    "nonstop",
    "normal",
    "notable",
    "noted",
    "noteworthy",
    "novel",
    "noxious",
    "numb",
    "nutritious",
    "nutty",
    "obedient",
    "obese",
    "oblong",
    "oily",
    "oblong",
    "obvious",
    "occasional",
    "odd",
    "oddball",
    "offbeat",
    "offensive",
    "official",
    "old",
    "old-fashioned",
    "only",
    "open",
    "optimal",
    "optimistic",
    "opulent",
    "orange",
    "orderly",
    "organic",
    "ornate",
    "ornery",
    "ordinary",
    "original",
    "other",
    "our",
    "outlying",
    "outgoing",
    "outlandish",
    "outrageous",
    "outstanding",
    "oval",
    "overcooked",
    "overdue",
    "overjoyed",
    "overlooked",
    "palatable",
    "pale",
    "paltry",
    "parallel",
    "parched",
    "partial",
    "passionate",
    "past",
    "pastel",
    "peaceful",
    "peppery",
    "perfect",
    "perfumed",
    "periodic",
    "perky",
    "personal",
    "pertinent",
    "pesky",
    "pessimistic",
    "petty",
    "phony",
    "physical",
    "piercing",
    "pink",
    "pitiful",
    "plain",
    "plaintive",
    "plastic",
    "playful",
    "pleasant",
    "pleased",
    "pleasing",
    "plump",
    "plush",
    "polished",
    "polite",
    "political",
    "pointed",
    "pointless",
    "poised",
    "poor",
    "popular",
    "portly",
    "posh",
    "positive",
    "possible",
    "potable",
    "powerful",
    "powerless",
    "practical",
    "precious",
    "present",
    "prestigious",
    "pretty",
    "precious",
    "previous",
    "pricey",
    "prickly",
    "primary",
    "prime",
    "pristine",
    "private",
    "prize",
    "probable",
    "productive",
    "profitable",
    "profuse",
    "proper",
    "proud",
    "prudent",
    "punctual",
    "pungent",
    "puny",
    "pure",
    "purple",
    "pushy",
    "putrid",
    "puzzled",
    "puzzling",
    "quaint",
    "qualified",
    "quarrelsome",
    "quarterly",
    "queasy",
    "querulous",
    "questionable",
    "quick",
    "quick-witted",
    "quiet",
    "quintessential",
    "quirky",
    "quixotic",
    "quizzical",
    "radiant",
    "ragged",
    "rapid",
    "rare",
    "rash",
    "raw",
    "recent",
    "reckless",
    "rectangular",
    "ready",
    "real",
    "realistic",
    "reasonable",
    "red",
    "reflecting",
    "regal",
    "regular",
    "reliable",
    "relieved",
    "remarkable",
    "remorseful",
    "remote",
    "repentant",
    "required",
    "respectful",
    "responsible",
    "repulsive",
    "revolving",
    "rewarding",
    "rich",
    "rigid",
    "right",
    "ringed",
    "ripe",
    "roasted",
    "robust",
    "rosy",
    "rotating",
    "rotten",
    "rough",
    "round",
    "rowdy",
    "royal",
    "rubbery",
    "rundown",
    "ruddy",
    "rude",
    "runny",
    "rural",
    "rusty",
    "sad",
    "safe",
    "salty",
    "same",
    "sandy",
    "sane",
    "sarcastic",
    "sardonic",
    "satisfied",
    "scaly",
    "scarce",
    "scared",
    "scary",
    "scented",
    "scholarly",
    "scientific",
    "scornful",
    "scratchy",
    "scrawny",
    "second",
    "secondary",
    "second-hand",
    "secret",
    "self-assured",
    "self-reliant",
    "selfish",
    "sentimental",
    "separate",
    "serene",
    "serious",
    "serpentine",
    "several",
    "severe",
    "shabby",
    "shadowy",
    "shady",
    "shallow",
    "shameful",
    "shameless",
    "sharp",
    "shimmering",
    "shiny",
    "shocked",
    "shocking",
    "shoddy",
    "short",
    "short-term",
    "showy",
    "shrill",
    "shy",
    "sick",
    "silent",
    "silky",
    "silly",
    "silver",
    "similar",
    "simple",
    "simplistic",
    "sinful",
    "single",
    "sizzling",
    "skeletal",
    "skinny",
    "sleepy",
    "slight",
    "slim",
    "slimy",
    "slippery",
    "slow",
    "slushy",
    "small",
    "smart",
    "smoggy",
    "smooth",
    "smug",
    "snappy",
    "snarling",
    "sneaky",
    "sniveling",
    "snoopy",
    "sociable",
    "soft",
    "soggy",
    "solid",
    "somber",
    "some",
    "spherical",
    "sophisticated",
    "sore",
    "sorrowful",
    "soulful",
    "soupy",
    "sour",
    "Spanish",
    "sparkling",
    "sparse",
    "specific",
    "spectacular",
    "speedy",
    "spicy",
    "spiffy",
    "spirited",
    "spiteful",
    "splendid",
    "spotless",
    "spotted",
    "spry",
    "square",
    "squeaky",
    "squiggly",
    "stable",
    "staid",
    "stained",
    "stale",
    "standard",
    "starchy",
    "stark",
    "starry",
    "steep",
    "sticky",
    "stiff",
    "stimulating",
    "stingy",
    "stormy",
    "straight",
    "strange",
    "steel",
    "strict",
    "strident",
    "striking",
    "striped",
    "strong",
    "studious",
    "stunning",
    "stupendous",
    "stupid",
    "sturdy",
    "stylish",
    "subdued",
    "submissive",
    "substantial",
    "subtle",
    "suburban",
    "sudden",
    "sugary",
    "sunny",
    "super",
    "superb",
    "superficial",
    "superior",
    "supportive",
    "sure-footed",
    "surprised",
    "suspicious",
    "svelte",
    "sweaty",
    "sweet",
    "sweltering",
    "swift",
    "sympathetic",
    "tall",
    "talkative",
    "tame",
    "tan",
    "tangible",
    "tart",
    "tasty",
    "tattered",
    "taut",
    "tedious",
    "teeming",
    "tempting",
    "tender",
    "tense",
    "tepid",
    "terrible",
    "terrific",
    "testy",
    "thankful",
    "that",
    "these",
    "thick",
    "thin",
    "third",
    "thirsty",
    "this",
    "thorough",
    "thorny",
    "those",
    "thoughtful",
    "threadbare",
    "thrifty",
    "thunderous",
    "tidy",
    "tight",
    "timely",
    "tinted",
    "tiny",
    "tired",
    "torn",
    "total",
    "tough",
    "traumatic",
    "treasured",
    "tremendous",
    "tragic",
    "trained",
    "tremendous",
    "triangular",
    "tricky",
    "trifling",
    "trim",
    "trivial",
    "troubled",
    "true",
    "trusting",
    "trustworthy",
    "trusty",
    "truthful",
    "tubby",
    "turbulent",
    "twin",
    "ugly",
    "ultimate",
    "unacceptable",
    "unaware",
    "uncomfortable",
    "uncommon",
    "unconscious",
    "understated",
    "unequaled",
    "uneven",
    "unfinished",
    "unfit",
    "unfolded",
    "unfortunate",
    "unhappy",
    "unhealthy",
    "uniform",
    "unimportant",
    "unique",
    "united",
    "unkempt",
    "unknown",
    "unlawful",
    "unlined",
    "unlucky",
    "unnatural",
    "unpleasant",
    "unrealistic",
    "unripe",
    "unruly",
    "unselfish",
    "unsightly",
    "unsteady",
    "unsung",
    "untidy",
    "untimely",
    "untried",
    "untrue",
    "unused",
    "unusual",
    "unwelcome",
    "unwieldy",
    "unwilling",
    "unwitting",
    "unwritten",
    "upbeat",
    "upright",
    "upset",
    "urban",
    "usable",
    "used",
    "useful",
    "useless",
    "utilized",
    "utter",
    "vacant",
    "vague",
    "vain",
    "valid",
    "valuable",
    "vapid",
    "variable",
    "vast",
    "velvety",
    "venerated",
    "vengeful",
    "verifiable",
    "vibrant",
    "vicious",
    "victorious",
    "vigilant",
    "vigorous",
    "villainous",
    "violet",
    "violent",
    "virtual",
    "virtuous",
    "visible",
    "vital",
    "vivacious",
    "vivid",
    "voluminous",
    "wan",
    "warlike",
    "warm",
    "warmhearted",
    "warped",
    "wary",
    "wasteful",
    "watchful",
    "waterlogged",
    "watery",
    "wavy",
    "wealthy",
    "weak",
    "weary",
    "webbed",
    "wee",
    "weekly",
    "weepy",
    "weighty",
    "weird",
    "welcome",
    "well-documented",
    "well-groomed",
    "well-informed",
    "well-lit",
    "well-made",
    "well-off",
    "well-to-do",
    "well-worn",
    "wet",
    "which",
    "whimsical",
    "whirlwind",
    "whispered",
    "white",
    "whole",
    "whopping",
    "wicked",
    "wide",
    "wide-eyed",
    "wiggly",
    "wild",
    "willing",
    "wilted",
    "winding",
    "windy",
    "winged",
    "wiry",
    "wise",
    "witty",
    "wobbly",
    "woeful",
    "wonderful",
    "wooden",
    "woozy",
    "wordy",
    "worldly",
    "worn",
    "worried",
    "worrisome",
    "worse",
    "worst",
    "worthless",
    "worthwhile",
    "worthy",
    "wrathful",
    "wretched",
    "writhing",
    "wrong",
    "wry",
    "yawning",
    "yearly",
    "yellow",
    "yellowish",
    "young",
    "youthful",
    "yummy",
    "zany",
    "zealous",
    "zesty",
    "zigzag",
]

ANIMALS = [
    "aardwolf",
    "admiral",
    "adouri",
    "african black crake",
    "african buffalo",
    "african bush squirrel",
    "african clawless otter",
    "african darter",
    "african elephant",
    "african fish eagle",
    "african ground squirrel",
    "african jacana",
    "african lion",
    "african lynx",
    "african pied wagtail",
    "african polecat",
    "african porcupine",
    "african red-eyed bulbul",
    "african skink",
    "african snake",
    "african wild cat",
    "african wild dog",
    "agama lizard",
    "agile wallaby",
    "agouti",
    "albatross",
    "alligator",
    "alpaca",
    "amazon parrot",
    "american alligator",
    "american badger",
    "american beaver",
    "american bighorn sheep",
    "american bison",
    "american black bear",
    "american buffalo",
    "american crow",
    "american marten",
    "american racer",
    "american virginia opossum",
    "american woodcock",
    "anaconda",
    "andean goose",
    "ant",
    "anteater",
    "antechinus",
    "antelope",
    "antelope ground squirrel",
    "arboral spiny rat",
    "arctic fox",
    "arctic ground squirrel",
    "arctic hare",
    "arctic lemming",
    "arctic tern",
    "argalis",
    "armadillo",
    "asian elephant",
    "asian false vampire bat",
    "asian foreset tortoise",
    "asian lion",
    "asian openbill",
    "asian red fox",
    "asian water buffalo",
    "asian water dragon",
    "asiatic jackal",
    "asiatic wild ass",
    "ass",
    "australian brush turkey",
    "australian magpie",
    "australian masked owl",
    "australian pelican",
    "australian sea lion",
    "australian spiny anteater",
    "avocet",
    "baboon",
    "badger",
    "bahama pintail",
    "bald eagle",
    "baleen whale",
    "banded mongoose",
    "bandicoot",
    "barasingha deer",
    "barbet",
    "bare-faced go away bird",
    "barking gecko",
    "barrows goldeneye",
    "bat",
    "bat-eared fox",
    "bateleur eagle",
    "bear",
    "beaver",
    "bee-eater",
    "beisa oryx",
    "bengal vulture",
    "bent-toed gecko",
    "bettong",
    "bird",
    "bison",
    "black and white colobus",
    "black-backed jackal",
    "black-backed magpie",
    "black bear",
    "blackbird",
    "blackbuck",
    "black-capped capuchin",
    "black-capped chickadee",
    "black-cheeked waxbill",
    "black-collared barbet",
    "black-crowned crane",
    "black-crowned night heron",
    "black curlew",
    "black-eyed bulbul",
    "black-faced kangaroo",
    "black-footed ferret",
    "black-fronted bulbul",
    "blackish oystercatcher",
    "black kite",
    "black-necked stork",
    "black rhinoceros",
    "blacksmith plover",
    "black spider monkey",
    "black swan",
    "black-tailed deer",
    "black-tailed prairie dog",
    "black-tailed tree creeper",
    "black-throated butcher bird",
    "black-throated cardinal",
    "black vulture",
    "black-winged stilt",
    "bleeding heart monkey",
    "blesbok",
    "bleu",
    "blue and gold macaw",
    "blue and yellow macaw",
    "blue-breasted cordon bleu",
    "blue catfish",
    "blue crane",
    "blue duck",
    "blue-faced booby",
    "blue-footed booby",
    "blue fox",
    "blue peacock",
    "blue racer",
    "blue shark",
    "blue-tongued lizard",
    "blue-tongued skink",
    "blue waxbill",
    "blue wildebeest",
    "boa",
    "boar",
    "boat-billed heron",
    "bobcat",
    "bohor reedbuck",
    "bonnet macaque",
    "bontebok",
    "booby",
    "bottle-nose dolphin",
    "boubou",
    "brazilian otter",
    "brazilian tapir",
    "brindled gnu",
    "brocket",
    "brolga crane",
    "brown and yellow marshbird",
    "brown antechinus",
    "brown brocket",
    "brown capuchin",
    "brown hyena",
    "brown lemur",
    "brown pelican",
    "brush-tailed bettong",
    "brush-tailed phascogale",
    "brush-tailed rat kangaroo",
    "buffalo",
    "bulbul",
    "bunting",
    "burmese black mountain tortoise",
    "burmese brown mountain tortoise",
    "burrowing owl",
    "bushbaby",
    "bushbuck",
    "bush dog",
    "bushpig",
    "bustard",
    "butterfly",
    "buttermilk snake",
    "caiman",
    "california sea lion",
    "camel",
    "campo flicker",
    "canada goose",
    "canadian river otter",
    "canadian tiger swallowtail butterfly",
    "cape barren goose",
    "cape clawless otter",
    "cape cobra",
    "cape fox",
    "cape raven",
    "cape starling",
    "cape white-eye",
    "cape wild cat",
    "capuchin",
    "capybara",
    "caracal",
    "caracara",
    "cardinal",
    "caribou",
    "carmine bee-eater",
    "carpet python",
    "carpet snake",
    "cat",
    "catfish",
    "cattle egret",
    "cereopsis goose",
    "chacma baboon",
    "chameleon",
    "cheetah",
    "chestnut weaver",
    "chickadee",
    "chilean flamingo",
    "chimpanzee",
    "chipmunk",
    "chital",
    "chuckwalla",
    "civet",
    "civet",
    "civet cat",
    "cliffchat",
    "coatimundi",
    "cobra",
    "cockatoo",
    "collared lemming",
    "collared lizard",
    "collared peccary",
    "colobus",
    "columbian rainbow boa",
    "comb duck",
    "common boubou shrike",
    "common brushtail possum",
    "common dolphin",
    "common duiker",
    "common eland",
    "common genet",
    "common goldeneye",
    "common green iguana",
    "common grenadier",
    "common langur",
    "common long-nosed armadillo",
    "common melba finch",
    "common mynah",
    "common nighthawk",
    "common palm civet",
    "common pheasant",
    "common raccoon",
    "common rhea",
    "common ringtail",
    "common seal",
    "common shelduck",
    "common turkey",
    "common wallaroo",
    "common waterbuck",
    "common wolf",
    "common wombat",
    "common zebra",
    "common zorro",
    "constrictor",
    "coot",
    "coqui francolin",
    "coqui partridge",
    "corella",
    "cormorant",
    "cottonmouth",
    "cougar",
    "cow",
    "coyote",
    "crab",
    "crab-eating fox",
    "crab-eating raccoon",
    "crake",
    "crane",
    "creeper",
    "crested barbet",
    "crested bunting",
    "crested porcupine",
    "crested screamer",
    "crimson-breasted shrike",
    "crocodile",
    "crow",
    "crowned eagle",
    "crowned hawk-eagle",
    "crown of thorns starfish",
    "cuis",
    "curlew",
    "currasow",
    "curve-billed thrasher",
    "dabchick",
    "dama wallaby",
    "dark-winged trumpeter",
    "darter",
    "darwin ground finch",
    "dassie",
    "deer",
    "defassa waterbuck",
    "desert kangaroo rat",
    "desert spiny lizard",
    "desert tortoise",
    "devil",
    "dik",
    "dingo",
    "dog",
    "dolphin",
    "dove",
    "downy woodpecker",
    "dragon",
    "dragonfly",
    "dromedary camel",
    "drongo",
    "duck",
    "duiker",
    "dunnart",
    "dusky gull",
    "dusky rattlesnake",
    "eagle",
    "eagle owl",
    "eastern boa constrictor",
    "eastern box turtle",
    "eastern cottontail rabbit",
    "eastern diamondback rattlesnake",
    "eastern dwarf mongoose",
    "eastern fox squirrel",
    "eastern grey kangaroo",
    "eastern indigo snake",
    "eastern quoll",
    "eastern white pelican",
    "echidna",
    "egret",
    "egyptian cobra",
    "egyptian goose",
    "egyptian viper",
    "egyptian vulture",
    "eland",
    "elegant crested tinamou",
    "elephant",
    "eleven-banded armadillo",
    "elk",
    "emerald green tree boa",
    "emerald-spotted wood dove",
    "emu",
    "eurasian badger",
    "eurasian beaver",
    "eurasian hoopoe",
    "eurasian red squirrel",
    "european badger",
    "european beaver",
    "european red squirrel",
    "european shelduck",
    "european spoonbill",
    "european stork",
    "european wild cat",
    "euro wallaby",
    "fairy penguin",
    "falcon",
    "fat-tailed dunnart",
    "feathertail glider",
    "feral rock pigeon",
    "ferret",
    "ferruginous hawk",
    "field flicker",
    "finch",
    "fisher",
    "flamingo",
    "flicker",
    "flightless cormorant",
    "flycatcher",
    "flying fox",
    "fork-tailed drongo",
    "four-horned antelope",
    "four-spotted skimmer",
    "four-striped grass mouse",
    "fowl",
    "fox",
    "francolin",
    "frilled dragon",
    "frilled lizard",
    "fringe-eared oryx",
    "frog",
    "frogmouth",
    "galah",
    "galapagos albatross",
    "galapagos dove",
    "galapagos hawk",
    "galapagos mockingbird",
    "galapagos penguin",
    "galapagos sea lion",
    "galapagos tortoise",
    "gaur",
    "gazelle",
    "gazer",
    "gecko",
    "gelada baboon",
    "gemsbok",
    "genet",
    "genoveva",
    "gerbil",
    "gerenuk",
    "giant anteater",
    "giant armadillo",
    "giant girdled lizard",
    "giant heron",
    "giant otter",
    "gila monster",
    "giraffe",
    "glider",
    "glossy ibis",
    "glossy starling",
    "gnu",
    "goanna lizard",
    "goat",
    "godwit",
    "golden brush-tailed possum",
    "golden eagle",
    "goldeneye",
    "golden jackal",
    "golden-mantled ground squirrel",
    "goliath heron",
    "gonolek",
    "goose",
    "gorilla",
    "gray duiker",
    "gray heron",
    "gray langur",
    "gray rhea",
    "great cormorant",
    "great egret",
    "greater adjutant stork",
    "greater blue-eared starling",
    "greater flamingo",
    "greater kudu",
    "greater rhea",
    "greater roadrunner",
    "greater sage grouse",
    "great horned owl",
    "great kiskadee",
    "great skua",
    "great white pelican",
    "grebe",
    "green-backed heron",
    "green heron",
    "green vine snake",
    "green-winged macaw",
    "green-winged trumpeter",
    "grenadier",
    "grey-footed squirrel",
    "grey fox",
    "grey heron",
    "greylag goose",
    "grey lourie",
    "grey mouse lemur",
    "grey phalarope",
    "griffon vulture",
    "grison",
    "grizzly bear",
    "groundhog",
    "ground legaan",
    "ground monitor",
    "grouse",
    "guanaco",
    "guerza",
    "gull",
    "gulls",
    "hanuman langur",
    "harbor seal",
    "hare",
    "hartebeest",
    "hawk",
    "hawk-eagle",
    "hawk-headed parrot",
    "hedgehog",
    "helmeted guinea fowl",
    "hen",
    "heron",
    "herring gull",
    "hippopotamus",
    "hoary marmot",
    "honey badger",
    "hoopoe",
    "hornbill",
    "horned lark",
    "horned puffin",
    "horned rattlesnake",
    "hottentot teal",
    "house crow",
    "house sparrow",
    "hudsonian godwit",
    "hummingbird",
    "huron",
    "hyena",
    "hyrax",
    "ibex",
    "ibis",
    "iguana",
    "impala",
    "indian giant squirrel",
    "indian jackal",
    "indian leopard",
    "indian mynah",
    "indian peacock",
    "indian porcupine",
    "indian red admiral",
    "indian star tortoise",
    "indian tree pie",
    "insect",
    "jabiru stork",
    "jacana",
    "jackal",
    "jackrabbit",
    "jaeger",
    "jaguar",
    "jaguarundi",
    "japanese macaque",
    "javanese cormorant",
    "javan gold-spotted mongoose",
    "jungle cat",
    "jungle kangaroo",
    "kaffir cat",
    "kafue flats lechwe",
    "kalahari scrub robin",
    "kangaroo",
    "kelp gull",
    "killer whale",
    "king cormorant",
    "kingfisher",
    "king vulture",
    "kinkajou",
    "kiskadee",
    "kite",
    "klipspringer",
    "knob-nosed goose",
    "koala",
    "komodo dragon",
    "kongoni",
    "kookaburra",
    "kori bustard",
    "kudu",
    "land iguana",
    "langur",
    "lappet-faced vulture",
    "lapwing",
    "large cormorant",
    "large-eared bushbaby",
    "lark",
    "laughing dove",
    "laughing kookaburra",
    "lava gull",
    "least chipmunk",
    "lechwe",
    "legaan",
    "lemming",
    "lemur",
    "leopard",
    "lesser double-collared sunbird",
    "lesser flamingo",
    "lesser masked weaver",
    "lesser mouse lemur",
    "lilac-breasted roller",
    "lily trotter",
    "lion",
    "little blue penguin",
    "little brown bat",
    "little brown dove",
    "little cormorant",
    "little grebe",
    "little heron",
    "lizard",
    "llama",
    "long-billed cockatoo",
    "long-billed corella",
    "long-crested hawk eagle",
    "long-finned pilot whale",
    "long-necked turtle",
    "long-nosed bandicoot",
    "long-tailed jaeger",
    "long-tailed skua",
    "long-tailed spotted cat",
    "lorikeet",
    "loris",
    "lory",
    "lourie",
    "lynx",
    "macaque",
    "macaw",
    "madagascar fruit bat",
    "madagascar hawk owl",
    "magellanic penguin",
    "magistrate black colobus",
    "magnificent frigate bird",
    "magpie",
    "malabar squirrel",
    "malachite kingfisher",
    "malagasy ground boa",
    "malay squirrel",
    "mallard",
    "malleefowl",
    "manatee",
    "mandras tree shrew",
    "mara",
    "marabou stork",
    "margay",
    "marine iguana",
    "marmot",
    "marshbird",
    "marten",
    "masked booby",
    "meerkat",
    "mexican beaded lizard",
    "mexican boa",
    "mexican wolf",
    "mississippi alligator",
    "moccasin",
    "mockingbird",
    "mocking cliffchat",
    "mongoose",
    "monitor",
    "monitor lizard",
    "monkey",
    "monster",
    "moorhen",
    "moose",
    "mouflon",
    "mountain duck",
    "mountain goat",
    "mountain lion",
    "mourning collared dove",
    "mouse",
    "mudskipper",
    "mule deer",
    "musk ox",
    "mynah",
    "native cat",
    "nelson ground squirrel",
    "neotropic cormorant",
    "netted rock dragon",
    "nighthawk",
    "nile crocodile",
    "nilgai",
    "nine-banded armadillo",
    "north american beaver",
    "north american porcupine",
    "north american red fox",
    "north american river otter",
    "northern elephant seal",
    "northern fur seal",
    "northern phalarope",
    "nubian bee-eater",
    "numbat",
    "nutcracker",
    "nuthatch",
    "nyala",
    "ocelot",
    "old world fruit bat",
    "olive baboon",
    "onager",
    "openbill",
    "openbill stork",
    "opossum",
    "orca",
    "oribi",
    "oriental short-clawed otter",
    "oriental white-backed vulture",
    "ornate rock dragon",
    "oryx",
    "osprey",
    "ostrich",
    "otter",
    "ovenbird",
    "owl",
    "ox",
    "oystercatcher",
    "paca",
    "pacific gull",
    "paddy heron",
    "pademelon",
    "painted stork",
    "pale-throated three-toed sloth",
    "pale white-eye",
    "palm squirrel",
    "pampa gray fox",
    "paradoxure",
    "parakeet",
    "parrot",
    "partridge",
    "peacock",
    "peccary",
    "pelican",
    "penguin",
    "peregrine falcon",
    "phalarope",
    "phascogale",
    "pheasant",
    "pie",
    "pied avocet",
    "pied butcher bird",
    "pied cormorant",
    "pied crow",
    "pied kingfisher",
    "pigeon",
    "pig-tailed macaque",
    "pine siskin",
    "pine snake",
    "pine squirrel",
    "pintail",
    "plains zebra",
    "platypus",
    "plover",
    "pocket gopher",
    "polar bear",
    "polecat",
    "porcupine",
    "possum",
    "potoroo",
    "prairie falcon",
    "praying mantis",
    "prehensile-tailed porcupine",
    "pronghorn",
    "puffin",
    "puku",
    "puma",
    "puna ibis",
    "purple grenadier",
    "purple moorhen",
    "pygmy possum",
    "python",
    "quail",
    "quoll",
    "rabbit",
    "raccoon",
    "raccoon dog",
    "racer",
    "racer snake",
    "radiated tortoise",
    "rainbow lory",
    "rat",
    "rattlesnake",
    "raven",
    "red and blue macaw",
    "red-billed buffalo weaver",
    "red-billed hornbill",
    "red-billed toucan",
    "red-billed tropic bird",
    "red-breasted cockatoo",
    "red-breasted nuthatch",
    "red brocket",
    "red-capped cardinal",
    "red-cheeked cordon bleu",
    "red deer",
    "red hartebeest",
    "red-headed woodpecker",
    "red howler monkey",
    "red kangaroo",
    "red-knobbed coot",
    "red lava crab",
    "red-legged pademelon",
    "red meerkat",
    "red-necked phalarope",
    "red-necked wallaby",
    "red phalarope",
    "red sheep",
    "red-shouldered glossy starling",
    "red squirrel",
    "red-tailed cockatoo",
    "red-tailed hawk",
    "red-tailed phascogale",
    "red-tailed wambenger",
    "red-winged blackbird",
    "red-winged hawk",
    "reedbuck",
    "reindeer",
    "rhea",
    "rhesus macaque",
    "rhesus monkey",
    "rhinoceros",
    "ring dove",
    "ring-necked pheasant",
    "ringtail",
    "ringtail",
    "ringtail cat",
    "ring-tailed coatimundi",
    "ring-tailed gecko",
    "ring-tailed lemur",
    "ring-tailed possum",
    "river wallaby",
    "roadrunner",
    "roan antelope",
    "robin",
    "rock dove",
    "roe deer",
    "roller",
    "roseate cockatoo",
    "roseat flamingo",
    "rose-ringed parakeet",
    "royal tern",
    "rufous-collared sparrow",
    "rufous tree pie",
    "russian dragonfly",
    "sable antelope",
    "sacred ibis",
    "saddle-billed stork",
    "sage grouse",
    "sage hen",
    "sally lightfoot crab",
    "salmon",
    "salmon pink bird eater tarantula",
    "sambar",
    "sandgrouse",
    "sandhill crane",
    "sandpiper",
    "sarus crane",
    "savanna baboon",
    "savanna fox",
    "savannah deer",
    "scaly-breasted lorikeet",
    "scarlet macaw",
    "scottish highland cow",
    "screamer",
    "sea birds",
    "seal",
    "secretary bird",
    "serval",
    "seven-banded armadillo",
    "shark",
    "sheathbill",
    "sheep",
    "shelduck",
    "short-beaked echidna",
    "short-nosed bandicoot",
    "shrew",
    "shrike",
    "sidewinder",
    "sifaka",
    "silver-backed fox",
    "silver-backed jackal",
    "silver gull",
    "siskin",
    "skimmer",
    "skink",
    "skua",
    "skunk",
    "slender-billed cockatoo",
    "slender loris",
    "sloth",
    "sloth bear",
    "small-clawed otter",
    "small indian mongoose",
    "small-spotted genet",
    "small-toothed palm civet",
    "snake",
    "snake-necked turtle",
    "snow goose",
    "snowy egret",
    "snowy owl",
    "snowy sheathbill",
    "sociable weaver",
    "sockeye salmon",
    "south african hedgehog",
    "south american meadowlark",
    "south american puma",
    "south american sea lion",
    "southern black-backed gull",
    "southern boubou",
    "southern brown bandicoot",
    "southern elephant seal",
    "southern ground hornbill",
    "southern hairy-nosed wombat",
    "southern lapwing",
    "southern right whale",
    "southern screamer",
    "southern sea lion",
    "southern tamandua",
    "southern white-crowned shrike",
    "sparrow",
    "spectacled caiman",
    "spider",
    "spoonbill",
    "sportive lemur",
    "spotted deer",
    "spotted hyena",
    "spotted-tailed quoll",
    "spotted wood sandpiper",
    "springbok",
    "springbuck",
    "springhare",
    "spurfowl",
    "spur-winged goose",
    "square-lipped rhinoceros",
    "squirrel",
    "squirrel glider",
    "stanley bustard",
    "stanley crane",
    "starfish",
    "starling",
    "steenbok",
    "steenbuck",
    "steller sea lion",
    "stick insect",
    "stilt",
    "stone sheep",
    "stork",
    "striated heron",
    "striped dolphin",
    "striped hyena",
    "striped skunk",
    "sugar glider",
    "sulfur-crested cockatoo",
    "sunbird",
    "sungazer",
    "sun gazer",
    "superb starling",
    "suricate",
    "swallow",
    "swallow-tail gull",
    "swamp deer",
    "swan",
    "tailless tenrec",
    "tamandua",
    "tammar wallaby",
    "tapir",
    "tarantula",
    "tasmanian devil",
    "tawny eagle",
    "tawny frogmouth",
    "tayra",
    "teal",
    "tenrec",
    "tern",
    "thirteen-lined squirrel",
    "thrasher",
    "three-banded plover",
    "tiger",
    "tiger cat",
    "tiger snake",
    "timber wolf",
    "tinamou",
    "toddy cat",
    "tokay gecko",
    "topi",
    "tortoise",
    "toucan",
    "tree porcupine",
    "tropical buckeye butterfly",
    "trotter",
    "trumpeter",
    "trumpeter swan",
    "tsessebe",
    "turaco",
    "turkey",
    "turkey vulture",
    "turtle",
    "two-banded monitor",
    "two-toed sloth",
    "two-toed tree sloth",
    "tyrant flycatcher",
    "uinta ground squirrel",
    "urial",
    "vervet monkey",
    "vicuna",
    "vine snake",
    "violet-crested turaco",
    "violet-eared waxbill",
    "viper",
    "vulture",
    "wagtail",
    "wallaby",
    "wallaroo",
    "wambenger",
    "wapiti",
    "warthog",
    "waterbuck",
    "water legaan",
    "water moccasin",
    "water monitor",
    "wattled crane",
    "waved albatross",
    "waxbill",
    "weaver",
    "weeper capuchin",
    "western bearded dragon",
    "western grey kangaroo",
    "western lowland gorilla",
    "western palm tanager",
    "western patch-nosed snake",
    "western pygmy possum",
    "western spotted skunk",
    "whale",
    "whip-tailed wallaby",
    "white-bellied sea eagle",
    "white-browed owl",
    "white-browed sparrow weaver",
    "white-cheeked pintail",
    "white-eye",
    "white-faced tree rat",
    "white-faced whistling duck",
    "white-fronted bee-eater",
    "white-fronted capuchin",
    "white-headed vulture",
    "white-lipped peccary",
    "white-mantled colobus",
    "white-necked raven",
    "white-necked stork",
    "white-nosed coatimundi",
    "white rhinoceros",
    "white-rumped vulture",
    "white spoonbill",
    "white stork",
    "white-tailed deer",
    "white-tailed jackrabbit",
    "white-throated kingfisher",
    "white-throated monitor",
    "white-throated robin",
    "white-throated toucan",
    "white-winged black tern",
    "white-winged dove",
    "white-winged tern",
    "wild boar",
    "wildebeest",
    "wild turkey",
    "wild water buffalo",
    "wolf",
    "wolf spider",
    "wombat",
    "woodchuck",
    "woodcock",
    "woodpecker",
    "wood pigeon",
    "woodrat",
    "woolly-necked stork",
    "worm snake",
    "woylie",
    "yak",
    "yellow baboon",
    "yellow-bellied marmot",
    "yellow-billed hornbill",
    "yellow-billed stork",
    "yellow-brown sungazer",
    "yellow-crowned night heron",
    "yellow-headed caracara",
    "yellow mongoose",
    "yellow-necked spurfowl",
    "yellow-rumped siskin",
    "yellow-throated sandgrouse",
    "zebra",
    "zorilla",
    "zorro",
]


class NameGenerator:
    @staticmethod
    def random_name():
        global ADJECTIVES, ANIMALS

        return f"{random.choice(ADJECTIVES)} {random.choice(ANIMALS)}"



from typing import TYPE_CHECKING, Dict, Sequence

if TYPE_CHECKING:
    from .command import Command


class CommandManager:
    """
    Manages available commands.
    """

    def __init__(self):
        self._commands: Dict[str, Command] = {}

    def register_command(self, command: "Command"):
        assert command.name not in self._commands, "Command by this name already registered"
        self._commands[command.name] = command

    def register_commands(self, commands: Sequence["Command"]):
        for command in commands:
            self.register_command(command)

    def unregister_command(self, command: "Command"):
        self._commands.pop(command.name, None)

    def unregister_commands(self, commands: Sequence["Command"]):
        for command in commands:
            self.unregister_command(command)

    def get_commands(self):
        return self._commands.values()



from typing import TYPE_CHECKING

from PySide6.QtGui import QKeySequence, Qt

from .menu import Menu, MenuEntry, MenuSeparator

if TYPE_CHECKING:
    from angrmanagement.ui.widgets.qlog_widget import QLogWidget


class LogMenu(Menu):
    def __init__(self, log_widget: "QLogWidget"):
        super().__init__("", parent=log_widget)

        self.entries.extend(
            [
                MenuEntry(
                    "&Copy selected content",
                    log_widget.copy_selected_messages,
                    shortcut=QKeySequence(Qt.CTRL | Qt.Key_C),
                ),
                MenuEntry("Copy selected message", log_widget.copy_selected),
                MenuEntry("Copy all content", log_widget.copy_all_messages),
                MenuEntry("Copy all messages", log_widget.copy_all),
                MenuSeparator(),
                MenuEntry("C&lear log", log_widget.clear_log),
            ]
        )



from PySide6.QtWidgets import QVBoxLayout

from angrmanagement.ui.widgets.qpatch_table import QPatchTable

from .view import BaseView


class PatchesView(BaseView):
    """
    View showing all patches.
    """

    def __init__(self, workspace, instance, default_docking_position, *args, **kwargs):
        super().__init__("patches", workspace, instance, default_docking_position, *args, **kwargs)

        self.base_caption = "Patches"
        self._patch_table: QPatchTable

        self._init_widgets()

        # Reload upon creation
        self.reload()

    def reload(self):
        self._patch_table.reload()

    #
    # Private methods
    #

    def _init_widgets(self):
        self._patch_table = QPatchTable(self.instance, self)

        layout = QVBoxLayout()
        layout.addWidget(self._patch_table)
        layout.setSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(layout)



import logging
from typing import TYPE_CHECKING

import angr.flirt

from .job import Job

if TYPE_CHECKING:
    from angrmanagement.data.instance import Instance

_l = logging.getLogger(name=__name__)


class FlirtSignatureRecognitionJob(Job):
    """
    Describes a job for using FLIRT signatures to recognize and match library functions embedded in a binary.
    """

    def __init__(self, on_finish=None):
        super().__init__(name="Applying FLIRT signatures", on_finish=on_finish)

    def _run(self, inst: "Instance"):
        if inst.project.arch.name.lower() in angr.flirt.FLIRT_SIGNATURES_BY_ARCH:
            inst.project.analyses.Flirt()
        else:
            _l.warning("No FLIRT signatures exist for architecture %s.", inst.project.arch.name)

    def __repr__(self):
        return "FlirtSignatureRecognitionJob"



import re
from typing import TYPE_CHECKING, Any

from angr.knowledge_plugins.cfg.memory_data import MemoryDataSort
from PySide6.QtCore import QAbstractTableModel, QSortFilterProxyModel, Qt
from PySide6.QtWidgets import QAbstractItemView, QHeaderView, QTableView

from angrmanagement.config import Conf
from angrmanagement.ui.dialogs.xref import XRefDialog
from angrmanagement.utils import filter_string_for_display

if TYPE_CHECKING:
    from angr.analyses.cfg.cfg_fast import MemoryData
    from PySide6.QtGui import QKeyEvent


class QStringModel(QAbstractTableModel):
    HEADER = ["Address", "Size (Bytes)", "Length", "String"]

    ADDRESS_COL = 0
    SIZE_COL = 1
    LENGTH_COL = 2
    STRING_COL = 3

    def __init__(self, cfg, func=None):
        super().__init__()

        self._cfg = cfg
        self._function = func
        self._xrefs = None

        self._values = None

    @property
    def cfg(self):
        return self._cfg

    @cfg.setter
    def cfg(self, v):
        self.beginResetModel()
        self._cfg = v
        self._values = None
        self.endResetModel()

    @property
    def xrefs(self):
        return self._xrefs

    @xrefs.setter
    def xrefs(self, v):
        self.beginResetModel()
        self._xrefs = v
        self._values = None
        self.endResetModel()

    @property
    def function(self):
        return self._function

    @function.setter
    def function(self, v):
        self.beginResetModel()
        self._function = v
        self._values = None
        self.endResetModel()

    def _get_all_string_memory_data(self):
        lst = []
        if self.cfg is None:
            return lst
        for v in self.cfg.memory_data.values():
            if v.sort in {MemoryDataSort.String, MemoryDataSort.UnicodeString}:
                if self._function is None:
                    lst.append(v)
                else:
                    xrefs = self._xrefs
                    if v.addr in xrefs.xrefs_by_dst:
                        for xref in xrefs.xrefs_by_dst[v.addr]:
                            if xref.block_addr in self._function.block_addrs_set:
                                lst.append(v)
                                break
        return lst

    @property
    def values(self):
        if self._values is None:
            self._values = self._get_all_string_memory_data()
        return self._values

    def __len__(self):
        return self.rowCount()

    def rowCount(self, parent=None) -> int:  # pylint: disable=unused-argument
        return len(self.values)

    def columnCount(self, parent=None) -> int:  # pylint: disable=unused-argument
        return len(self.HEADER)

    def headerData(self, section, orientation, role=None) -> Any:  # pylint: disable=unused-argument
        if role == Qt.DisplayRole:
            if section < len(self.HEADER):
                return self.HEADER[section]
        elif role == Qt.InitialSortOrderRole:
            return Qt.AscendingOrder

        return None

    def data(self, index, role=None) -> Any:
        if not index.isValid():
            return None

        row = index.row()
        if row >= len(self.values):
            return None

        col = index.column()
        v = self.values[row]

        if role == Qt.DisplayRole:
            return self._get_column_text(v, col)
        elif role == Qt.FontRole:
            return Conf.tabular_view_font
        return None

    def sort(self, column, order=None) -> Any:
        self.layoutAboutToBeChanged.emit()

        self._values = sorted(
            self.values,
            key=lambda x: self._get_column_data(x, column),
            reverse=order == Qt.DescendingOrder,
        )
        self.layoutChanged.emit()

    def _get_column_text(self, v: "MemoryData", col: int):
        if col < len(self.HEADER):
            data = self._get_column_data(v, col)
            if col == self.ADDRESS_COL and isinstance(data, int):
                return f"{data:x}"
            return data

    @staticmethod
    def _get_decoded_string_content(md: "MemoryData") -> str:
        sort_to_encoding = {MemoryDataSort.String: "utf-8", MemoryDataSort.UnicodeString: "utf_16_le"}
        return md.content.decode(sort_to_encoding[md.sort])

    def _get_column_data(self, v: "MemoryData", col: int) -> Any:
        mapping = {
            self.ADDRESS_COL: lambda x: x.addr,
            self.SIZE_COL: lambda x: x.size,
            self.LENGTH_COL: lambda x: len(self._get_decoded_string_content(x)) if x.content is not None else "<ERROR>",
            self.STRING_COL: lambda x: filter_string_for_display(self._get_decoded_string_content(x))
            if x.content is not None
            else "<ERROR>",
        }

        if col in mapping:
            return mapping[col](v)
        return None


class QStringTable(QTableView):
    def __init__(self, instance, parent, selection_callback=None):
        super().__init__(parent)

        self._instance = instance
        self._selected = selection_callback
        self._filter = None

        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setShowGrid(False)
        self.verticalHeader().setVisible(False)
        self.verticalHeader().setDefaultSectionSize(24)
        self.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)

        self._model = QStringModel(None)
        self._proxy = QSortFilterProxyModel(self)
        self._proxy.setSourceModel(self._model)
        self._proxy.setFilterCaseSensitivity(Qt.CaseInsensitive)
        self.setModel(self._proxy)

        self.setSortingEnabled(True)
        self.setSelectionMode(QAbstractItemView.SingleSelection)

        # let the last column (string) fill table width
        self.horizontalHeader().setSectionResizeMode(QHeaderView.Fixed)
        self.horizontalHeader().setSectionResizeMode(QStringModel.STRING_COL, QHeaderView.Stretch)

        self.doubleClicked.connect(self._on_string_selected)

    #
    # Properties
    #

    @property
    def cfg(self):
        return self._model.cfg

    @cfg.setter
    def cfg(self, v):
        self._model.cfg = v
        self.fast_resize()

    @property
    def xrefs(self):
        return self._model.xrefs

    @xrefs.setter
    def xrefs(self, v):
        self._model.xrefs = v

    @property
    def function(self):
        return self._model.function

    @function.setter
    def function(self, v):
        self._model.function = v
        self.fast_resize()

    @property
    def filter_string(self):
        return self._filter

    @filter_string.setter
    def filter_string(self, v):
        self._filter = v
        if isinstance(v, re.Pattern):
            self._proxy.setFilterRegExp(self._filter.pattern)
        else:
            self._proxy.setFilterWildcard(self._filter)
        self._proxy.setFilterKeyColumn(2)

    #
    # Public methods
    #

    def fast_resize(self):
        self.setVisible(False)
        self.resizeColumnsToContents()
        self.setVisible(True)

    #
    # Event handlers
    #

    def _on_string_selected(self, model_index):
        model_index = self._proxy.mapToSource(model_index)
        selected_index = model_index.row()
        if self._model is None:
            return
        selected_item = self._model.values[selected_index] if 0 <= selected_index < len(self._model.values) else None

        if self._selected is not None:
            self._selected(selected_item)

    def keyPressEvent(self, event: "QKeyEvent") -> None:
        if event.key() == Qt.Key_X:
            # xrefs
            if self._model is None:
                return

            selected_rows = list(self.selectionModel().selectedRows())
            if len(selected_rows) == 1:
                model_index = self._proxy.mapToSource(selected_rows[0])
                selected_index = model_index.row()
                if 0 <= selected_index < len(self._model.values):
                    selected_item: MemoryData = self._model.values[selected_index]
                    dialog = XRefDialog(
                        addr=selected_item.addr,
                        dst_addr=selected_item.addr,
                        xrefs_manager=self.xrefs,
                        instance=self._instance,
                        parent=self,
                    )
                    dialog.exec_()
            return

        return super().keyPressEvent(event)



from angrmanagement.utils.graph import to_supergraph


def edge_qualifies(data):
    return data["type"] not in ("call", "return_from_call")


class FunctionGraph:
    def __init__(self, function, exception_edges=True):
        self.function = function
        self.exception_edges = exception_edges
        self.edges = None
        self._supergraph = None

    def clear_cache(self):
        self._supergraph = None
        self.edges = None

    @property
    def supergraph(self):
        if self._supergraph is not None:
            return self._supergraph

        self._supergraph = to_supergraph(self.function.transition_graph_ex(exception_edges=self.exception_edges))
        self.edges = [
            (str(from_.addr), str(to.addr))
            for (from_, to, data) in self._supergraph.edges(data=True)
            if edge_qualifies(data)
        ]

        return self._supergraph



from PySide6.QtCore import Qt
from PySide6.QtGui import QKeySequence

from .menu import Menu, MenuEntry, MenuSeparator


class AnalyzeMenu(Menu):
    def __init__(self, main_window):
        super().__init__("&Analyze", parent=main_window)
        self.entries.extend(
            [
                MenuEntry("&Run Analysis...", main_window.run_analysis, shortcut=QKeySequence(Qt.Key_F4)),
                MenuSeparator(),
                MenuEntry("&Decompile", main_window.decompile_current_function, shortcut=QKeySequence(Qt.Key_F5)),
                MenuEntry(
                    "View in Proximity &Browser",
                    main_window.view_proximity_for_current_function,
                    shortcut=QKeySequence(Qt.CTRL | Qt.Key_B),
                ),
                MenuEntry("&Interact", main_window.interact, shortcut=QKeySequence(Qt.Key_F6)),
            ],
        )



from typing import TYPE_CHECKING

from angr.analyses.decompiler.structured_codegen.c import (
    CClosingObject,
    CConstant,
    CFunction,
    CFunctionCall,
    CStructField,
    CVariable,
)
from angr.sim_type import SimType
from PySide6.QtGui import QTextDocument
from PySide6.QtWidgets import QPlainTextDocumentLayout

from angrmanagement.config import Conf

if TYPE_CHECKING:
    from angr.analyses.decompiler.structured_codegen.base import BaseStructuredCodeGenerator


class QCodeDocument(QTextDocument):
    """
    QTextDocument class for pseudocode generated by StructuredCodeGenerator analysis in angr.
    """

    def __init__(self, codegen):
        super().__init__()

        self._codegen: BaseStructuredCodeGenerator = codegen

        # default font
        self.setDefaultFont(Conf.code_font)

        self.setPlainText(self._codegen.text)
        self.setDocumentLayout(QPlainTextDocumentLayout(self))

    @property
    def posmap(self):
        """

        :return:
        :rtype:     Optional[PositionMapping]
        """
        if self._codegen is None:
            return None
        return self._codegen.map_pos_to_node

    @property
    def nodemap(self):
        if self._codegen is None:
            return None
        return self._codegen.map_ast_to_pos

    def get_node_at_position(self, pos):
        if self._codegen is not None and self._codegen.map_pos_to_node is not None:
            n = self._codegen.map_pos_to_node.get_node(pos)
            if n is None:
                n = self._codegen.map_pos_to_node.get_node(pos - 1)
            return n

        return None

    def get_stmt_node_at_position(self, pos):
        """
        Iteratively finds the first valid node inside the GUI display that is not None.
        Finds the node based on the postion given (usually related to the mouse location).
        The function can return any valid Cxxx class inside the angr decompiler.

        Algorithm: O(n)
        This algorithm will search the position map by alternating between extended it's left and right search
        until we either hit the EOL or newline on the left and right. We stop either side searching once we
        hit a newline or EOL as well. In addition, we keep a special case for when the cursor is off the screen
        to simply put it back on the screen.

        :param pos:
        :return:
        """

        if self._codegen is None or self._codegen.map_pos_to_addr is None:
            return None

        n = self._codegen.map_pos_to_addr.get_node(pos)

        # if we can't find a node at the current position, start the algorithm search
        # from the left and right iteratively.
        if n is None:
            # special case where cursor is off the screen, reposition to before the end
            length = len(self._codegen.text) - 4 if pos >= len(self._codegen.text) - 2 else pos - 1

            r = pos + 1
            inc_l = not self._pos_is_newline_or_oob(length)
            inc_r = not self._pos_is_newline_or_oob(r)

            # iterate until we hit start or end of document
            while inc_l or inc_r:
                # continue left search if we are still at a valid char
                if inc_l:
                    n = self._codegen.map_pos_to_addr.get_node(length)
                    if n is not None:
                        break
                    length -= 1
                    inc_l = not self._pos_is_newline_or_oob(length)

                # continue right search if we are still at a valid char
                if inc_r:
                    n = self._codegen.map_pos_to_addr.get_node(r)
                    if n is not None:
                        break
                    r += 1
                    inc_r = not self._pos_is_newline_or_oob(r)

        return n

    def find_closest_node_pos(self, ins_addr):
        return self._codegen.map_addr_to_pos.get_nearest_pos(ins_addr)

    def find_related_text_chunks(self, node):
        if self._codegen is None or self._codegen.map_ast_to_pos is None:
            return None

        if isinstance(node, CConstant):
            starts = self._codegen.map_ast_to_pos.get(node.value, None)
            if starts is None:
                return []

        elif isinstance(node, CVariable):
            if node.unified_variable is not None:
                starts = self._codegen.map_ast_to_pos.get(node.unified_variable, None)
            else:
                starts = self._codegen.map_ast_to_pos.get(node.variable, None)
            if starts is None:
                return []

        elif isinstance(node, CFunctionCall):
            starts = self._codegen.map_ast_to_pos.get(
                node.callee_func if node.callee_func is not None else node.callee_target, None
            )
            if starts is None:
                return []

        elif isinstance(node, CStructField):
            key = (node.struct_type, node.offset)
            starts = self._codegen.map_ast_to_pos.get(key, None)

            if starts is None:
                return []

        elif isinstance(node, CClosingObject):
            starts = self._codegen.map_ast_to_pos.get(node, None)

            if starts is None:
                return []

        elif isinstance(node, CFunction):
            starts = self._codegen.map_ast_to_pos.get(node, None)

            if starts is None:
                return []

        elif isinstance(node, SimType):
            starts = self._codegen.map_ast_to_pos.get(node, None)

            if starts is None:
                return []

        else:
            # Unsupported
            return []

        chunks = []
        for start in starts:
            elem = self._codegen.map_pos_to_node.get_element(start)
            if elem is None:
                continue
            chunks.append((elem.start, elem.length + elem.start))
        return chunks

    #
    #   Private Helper Functions
    #

    def _pos_is_newline_or_oob(self, pos: int):
        """
        Checks if a position is a newline or out of bounds of the
        text generated from the decompiler.

        :param pos:
        :return:
        """
        if pos >= len(self._codegen.text):
            return True

        return self._codegen.text[pos] == "\n"



from threading import Thread
from typing import Any, Callable, Tuple


def start_daemon_thread(target: Callable[..., Any], name: str, args: Tuple[Any] = None) -> Thread:
    """
    Start a daemon thread.
    """
    t = Thread(target=target, name=name, args=args if args else ())
    t.daemon = True
    t.start()
    return t



import os
from typing import Optional

from PySide6.QtCore import QStandardPaths

from .config_manager import ConfigurationManager

# Global configuration manager instance
config_dir: str = QStandardPaths.locate(
    QStandardPaths.StandardLocation.AppConfigLocation, "angr-management", QStandardPaths.LocateOption.LocateDirectory
)
if config_dir == "":
    system_config_dir = QStandardPaths.writableLocation(QStandardPaths.StandardLocation.AppConfigLocation)
    if system_config_dir == "":
        print("Could not find configuration directory - settings will not be saved")
        config_dir = ""
    config_dir = os.path.join(system_config_dir, "angr-management")
    os.makedirs(config_dir, exist_ok=True)

config_path: Optional[str]
if config_dir != "":
    config_path = os.path.join(config_dir, "config.toml")
    try:
        Conf = ConfigurationManager.parse_file(config_path)
    except FileNotFoundError:
        Conf = ConfigurationManager()
else:
    config_path = None
    print("Could not find configuration directory - settings will not be saved")
    Conf = ConfigurationManager()


def save_config():
    if config_path is None:
        return
    Conf.save_file(config_path)


APP_LOCATION = str(os.path.join(os.path.dirname(os.path.realpath(__file__)), ".."))
PLUGIN_PATH = str(os.path.join(APP_LOCATION, "plugins"))
RES_LOCATION = str(os.path.join(APP_LOCATION, "resources"))
IMG_LOCATION = str(os.path.join(RES_LOCATION, "images"))
FONT_LOCATION = str(os.path.join(RES_LOCATION, "fonts"))
THEME_LOCATION = str(os.path.join(RES_LOCATION, "themes"))



"""
All functions in this module return a list of mixed BasePlugin class and exception objects to indicate
either the success or failure of a load.

This file intentionally does not reuse angr.misc.autoimport since it has additional design goals (managing exceptions,
loading out-of-tree files).
"""
import logging

from .base_plugin import BasePlugin
from .load import (
    load_default_plugins,
    load_plugin_description,
    load_plugin_descriptions_from_dir,
    load_plugins_from_dir,
    load_plugins_from_file,
    load_plugins_from_module,
    load_plugins_from_package,
    load_plugins_from_vars,
)
from .plugin_description import PluginDescription
from .plugin_manager import PluginManager

log = logging.getLogger(__name__)


__all__ = [
    "BasePlugin",
    "PluginDescription",
    "PluginManager",
    "load_plugin_descriptions_from_dir",
    "load_plugin_description",
    "load_default_plugins",
    "load_plugins_from_dir",
    "load_plugins_from_package",
    "load_plugins_from_file",
    "load_plugins_from_module",
    "load_plugins_from_vars",
]



from PySide6.QtCore import QRectF, Qt
from PySide6.QtWidgets import QApplication, QGraphicsSimpleTextItem

from .qgraph_object import QCachedGraphicsItem


class QVariable(QCachedGraphicsItem):
    IDENT_LEFT_PADDING = 5
    OFFSET_LEFT_PADDING = 12

    def __init__(self, instance, disasm_view, variable, config, infodock, parent=None):
        super().__init__(parent=parent)

        # initialization
        self.instance = instance
        self.disasm_view = disasm_view
        self.variable = variable
        self._config = config
        self.infodock = infodock

        self._variable_name = None
        self._variable_name_item: QGraphicsSimpleTextItem = None
        self._variable_ident = None
        self._variable_ident_item: QGraphicsSimpleTextItem = None
        self._variable_offset = None
        self._variable_offset_item: QGraphicsSimpleTextItem = None

        self._init_widgets()

    @property
    def selected(self):
        return self.infodock.is_variable_selected(self.variable)

    #
    # Public methods
    #

    def paint(self, painter, option, widget):  # pylint: disable=unused-argument
        # Background
        if self.selected:
            painter.setPen(self._config.disasm_view_operand_select_color)
            painter.setBrush(self._config.disasm_view_operand_select_color)
            painter.drawRect(0, 0, self.width, self.height)

    def refresh(self):
        super().refresh()

        if self._variable_ident_item is not None:
            self._variable_ident_item.setVisible(self.disasm_view.show_variable_identifier)

        self._layout_items_and_update_size()

    #
    # Events
    #

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.infodock.toggle_variable_selection(
                self.variable,
                unique=QApplication.keyboardModifiers() != Qt.ControlModifier,
            )
        else:
            super().mousePressEvent(event)

    #
    # Private methods
    #

    def _init_widgets(self):
        # variable name
        self._variable_name = "" if not self.variable.name else self.variable.name
        self._variable_name_item = QGraphicsSimpleTextItem(self._variable_name, self)
        self._variable_name_item.setFont(self._config.disasm_font)
        self._variable_name_item.setBrush(self._config.disasm_view_variable_label_color)

        # variable ident
        self._variable_ident = "<%s>" % ("" if not self.variable.ident else self.variable.ident)
        self._variable_ident_item = QGraphicsSimpleTextItem(self._variable_ident, self)
        self._variable_ident_item.setFont(self._config.disasm_font)
        self._variable_ident_item.setBrush(self._config.disasm_view_variable_ident_color)
        self._variable_ident_item.setVisible(self.disasm_view.show_variable_identifier)

        # variable offset
        self._variable_offset = "%#x" % self.variable.offset
        self._variable_offset_item = QGraphicsSimpleTextItem(self._variable_offset, self)
        self._variable_offset_item.setFont(self._config.disasm_font)
        self._variable_offset_item.setBrush(self._config.disasm_view_variable_offset_color)

        self._layout_items_and_update_size()

    def _layout_items_and_update_size(self):
        x, y = 0, 0

        # variable name
        self._variable_name_item.setPos(x, y)
        x += self._variable_name_item.boundingRect().width() + self.IDENT_LEFT_PADDING

        if self.disasm_view.show_variable_identifier:
            # identifier
            x += self.IDENT_LEFT_PADDING
            self._variable_ident_item.setPos(x, y)
            x += self._variable_ident_item.boundingRect().width()

        # variable offset
        x += self.OFFSET_LEFT_PADDING
        self._variable_offset_item.setPos(x, y)
        x += self._variable_offset_item.boundingRect().width()

        self._width = x
        self._height = self._variable_name_item.boundingRect().height()
        self.recalculate_size()

    def _boundingRect(self):
        return QRectF(0, 0, self._width, self._height)



from PySide6.QtCore import Qt
from PySide6.QtGui import QKeySequence

from .menu import Menu, MenuEntry, MenuSeparator


class HelpMenu(Menu):
    def __init__(self, main_window):
        super().__init__("&Help", parent=main_window)

        self.entries.extend(
            [
                MenuEntry("&Documentation", main_window.open_doc_link, shortcut=QKeySequence(Qt.ALT | Qt.Key_H)),
                MenuSeparator(),
                MenuEntry("About angr...", main_window.open_about_dialog),
            ]
        )



import logging
from typing import TYPE_CHECKING, Any, Optional, Set, Union

from angr.analyses.decompiler.structured_codegen import DummyStructuredCodeGenerator
from angr.analyses.decompiler.structured_codegen.c import CConstant, CFunctionCall, CStructuredCodeGenerator, CVariable
from angr.sim_variable import SimMemoryVariable
from PySide6.QtCore import Qt
from PySide6.QtGui import QTextCursor
from PySide6.QtWidgets import QComboBox, QDockWidget, QFrame, QHBoxLayout, QMainWindow, QTextEdit, QVBoxLayout, QWidget

from angrmanagement.config import Conf
from angrmanagement.data.jobs import DecompileFunctionJob, VariableRecoveryJob
from angrmanagement.data.object_container import ObjectContainer
from angrmanagement.logic.disassembly import JumpHistory
from angrmanagement.ui.dialogs.jumpto import JumpTo
from angrmanagement.ui.documents import QCodeDocument
from angrmanagement.ui.toolbars import NavToolbar
from angrmanagement.ui.widgets.qccode_edit import QCCodeEdit
from angrmanagement.ui.widgets.qdecomp_options import QDecompilationOptions

from .disassembly_view import DisassemblyView
from .view import BaseView

if TYPE_CHECKING:
    from angr.knowledge_plugins.functions.function import Function

log = logging.getLogger(__name__)


class CodeView(BaseView):
    """
    A view to display pseudocode or source code. You should control this view by manipulating and observing its four
    ObjectContainers: .addr, .current_node, .codegen, and .function.
    """

    FUNCTION_SPECIFIC_VIEW = True

    def __init__(self, workspace, instance, default_docking_position, *args, **kwargs):
        super().__init__("pseudocode", workspace, instance, default_docking_position, *args, **kwargs)

        self.base_caption = "Pseudocode"

        self._function: Union[ObjectContainer, Function] = ObjectContainer(None, "The function to decompile")
        self.current_node = ObjectContainer(None, "Current selected C-code node")
        self.addr: Union[ObjectContainer, int] = ObjectContainer(0, "Current cursor address")
        self.codegen: Union[ObjectContainer, CStructuredCodeGenerator] = ObjectContainer(
            None, "The currently-displayed codegen object"
        )

        self._last_function: Optional[Function] = None
        self._textedit: Optional[QCCodeEdit] = None
        self._doc: Optional[QCodeDocument] = None
        self._options: Optional[QDecompilationOptions] = None
        self.jump_history: JumpHistory = JumpHistory()
        self._nav_toolbar: Optional[NavToolbar] = None
        self._view_selector: Optional[QComboBox] = None

        self.vars_must_struct: Set[str] = set()

        self._init_widgets()

        self._textedit.cursorPositionChanged.connect(self._on_cursor_position_changed)
        self._textedit.selectionChanged.connect(self._on_cursor_position_changed)
        self._textedit.mouse_double_clicked.connect(self._on_mouse_doubleclicked)
        self._function.am_subscribe(self._on_new_function)
        self.codegen.am_subscribe(self._on_codegen_changes)
        self.addr.am_subscribe(self._on_new_addr)
        self.current_node.am_subscribe(self._on_new_node)

    def _focus_core(self, focus: bool, focus_addr: int):
        if focus:
            self.focus()
        if focus_addr is not None:
            self.addr.am_obj = focus_addr
            self.addr.am_event(focus=True)

    #
    # Properties
    #

    @property
    def textedit(self):
        return self._textedit

    @property
    def document(self):
        return self._doc

    @property
    def function(self) -> ObjectContainer:
        return self._function

    @function.setter
    def function(self, v):
        if v is not self._function.am_obj:
            self._function.am_obj = v
            self._function.am_event(focus=True)

    #
    # Public methods
    #

    def reload(self):
        if self.instance.project.am_none:
            return
        self._options.reload(force=True)
        self.vars_must_struct = set()

    def decompile(
        self,
        clear_prototype: bool = True,
        focus=False,
        focus_addr=None,
        flavor="pseudocode",
        reset_cache: bool = False,
        regen_clinic: bool = True,
    ):
        if self._function.am_none:
            return

        if clear_prototype:
            # clear the existing function prototype
            self._function.prototype = None
            self._function.ran_cca = False

        if reset_cache:
            self.instance.kb.structured_code.discard((self._function.addr, flavor))
            variables = self.instance.pseudocode_variable_kb.variables
            if variables.has_function_manager(self._function.addr):
                del variables[self._function.addr]

        def decomp_ready():
            # this code is _partially_ duplicated from _on_new_function. be careful!
            available = self.instance.kb.structured_code.available_flavors(self._function.addr)
            self._update_available_views(available)
            if available:
                chosen_flavor = flavor if flavor in available else available[0]
                self.codegen.am_obj = self.instance.kb.structured_code[(self._function.addr, chosen_flavor)].codegen
                self.codegen.am_event(already_regenerated=True)
                self._focus_core(focus, focus_addr)
                if focus_addr is not None:
                    self.jump_history.record_address(focus_addr)
                else:
                    self.jump_history.record_address(self._function.am_obj.addr)

        def decomp():
            job = DecompileFunctionJob(
                self._function.am_obj,
                cfg=self.instance.cfg,
                options=self._options.option_and_values,
                optimization_passes=self._options.selected_passes,
                peephole_optimizations=self._options.selected_peephole_opts,
                vars_must_struct=self.vars_must_struct,
                on_finish=decomp_ready,
                blocking=True,
                regen_clinic=regen_clinic,
            )
            self.instance.add_job(job)

        if self._function.ran_cca is False:
            # run calling convention analysis for this function
            if self.instance._analysis_configuration:
                options = self.instance._analysis_configuration["varec"].to_dict()
            else:
                options = {}
            options["workers"] = 0
            varrec_job = VariableRecoveryJob(**options, on_finish=decomp, func_addr=self._function.addr)
            self.instance.add_job(varrec_job)
        else:
            decomp()

    def highlight_chunks(self, chunks):
        extra_selections = []
        for start, end in chunks:
            sel = QTextEdit.ExtraSelection()
            sel.cursor = self._textedit.textCursor()
            sel.cursor.setPosition(start)
            sel.cursor.setPosition(end, QTextCursor.KeepAnchor)
            sel.format.setBackground(Conf.pseudocode_highlight_color)
            extra_selections.append(sel)
        self._textedit.setExtraSelections(extra_selections)

    #
    # Event callbacks
    #

    def _on_new_addr(self, already_moved=False, focus=False, **kwargs):  # pylint: disable=unused-argument
        if already_moved:
            return

        # get closest node for ins
        new_text_pos = self._doc.find_closest_node_pos(self.addr.am_obj) if self._doc is not None else None

        if new_text_pos is not None:
            # set the new cursor position
            textedit = self.textedit
            cursor = textedit.textCursor()
            cursor.setPosition(new_text_pos)
            textedit.setTextCursor(cursor)
            if focus:
                textedit.setFocus()
                self._focus_core(True, None)
        else:
            # try to find the right function
            block_addr, _ = self.instance.cfb.floor_item(self.addr.am_obj)
            block = self.instance.cfg.get_any_node(block_addr)
            if block is not None:
                func = self.instance.kb.functions[block.function_address]
                if func is not self._function.am_obj:
                    self._function.am_obj = func
                    self._function.am_event(focus_addr=self.addr.am_obj, focus=focus)
                else:
                    log.error(
                        "There is a block which is in the current function but find_closest_node_pos failed on it"
                    )

    def _on_new_node(self, **kwargs):  # pylint: disable=unused-argument
        self.addr.am_obj = self._textedit.get_src_to_inst()
        self.addr.am_event(already_moved=True)

    # pylint: disable=unused-argument
    def _on_codegen_changes(self, already_regenerated=False, event: Optional[str] = None, **kwargs):
        """
        The callback function that triggers an update of the codegen.

        :param already_regenerated: True if we only want to re-render the text and do not intend to change the text.
                                    Setting it to True will ignore `event` and other parameters.
        :param event:               The event to perform. For example, "retype_variable" will cause a re-flow of
                                    variable types. Leaving it unspecified will lead to regeneration of text (which is
                                    the default behavior).
                                    Supported events: retype_variable
        :param kwargs:              Keyword arguments that are required in each event.
        :return:
        """
        if self.codegen.am_none:
            return

        old_lineno: Optional[int] = None
        old_node: Optional[Any] = None
        old_font = None
        if self._last_function is self._function.am_obj and self._doc is not None:
            # we are re-rendering the current function (e.g., triggered by a node renaming). the cursor should stay at
            # the same node
            old_cursor: QTextCursor = self._textedit.textCursor()
            old_pos = old_cursor.position()
            old_lineno = old_cursor.blockNumber()
            old_font = self._textedit.font()
            # TODO: If multiple instances of the node is referenced at the same line, we will always select the first
            #  node. We need to fix this by counting which node is selected.
            old_node = self._doc.get_node_at_position(old_pos)

        self._view_selector.setCurrentText(self.codegen.flavor)

        if already_regenerated:
            # do not regenerate text
            pass
        else:
            if event == "retype_variable":
                dec = self.instance.project.analyses.Decompiler(
                    self._function.am_obj, variable_kb=self.instance.pseudocode_variable_kb, decompile=False
                )
                dec_cache = self.instance.kb.structured_code[(self._function.addr, "pseudocode")]
                new_codegen = dec.reflow_variable_types(
                    dec_cache.type_constraints,
                    dec_cache.var_to_typevar or {},
                    dec_cache.codegen,
                )
                # update the cache
                dec_cache.codegen = new_codegen

                # update self
                self.codegen.am_obj = new_codegen

            # regenerate text in the end
            self.codegen.regenerate_text()

        self._options.dirty = False
        if self._doc is None:
            self._doc = QCodeDocument(self.codegen)
            self._textedit.setDocument(self._doc)
        else:
            # only update the text. do not set a new document. this avoids scrolling the textedit up and down
            self._doc._codegen = self.codegen
            self._doc.setPlainText(self.codegen.text)

        if old_lineno is not None:
            the_block = self._doc.findBlockByNumber(old_lineno)
            new_cursor: QTextCursor = QTextCursor(the_block)
            self._textedit.setFont(old_font)
            if old_node is not None:
                # find the first node starting from the current position
                for pos in self.codegen.map_pos_to_node._posmap.irange(
                    minimum=new_cursor.position(), maximum=new_cursor.position() + the_block.length()
                ):
                    elem = self.codegen.map_pos_to_node._posmap[pos]
                    if elem.obj is old_node:
                        new_cursor.setPosition(pos)
            self._textedit.setTextCursor(new_cursor)

        if self.codegen.flavor == "pseudocode":
            self._options.show()
        else:
            self._options.hide()

    def _on_new_function(self, focus=False, focus_addr=None, flavor=None, **kwargs):  # pylint: disable=unused-argument
        # sets a new function. extra args are used in case this operation requires waiting for the decompiler
        if flavor is None:
            flavor = "pseudocode" if self.codegen.am_none else self.codegen.flavor

        if not self.codegen.am_none and self._last_function is self._function.am_obj:
            self._focus_core(focus, focus_addr)
            return
        available = self.instance.kb.structured_code.available_flavors(self._function.addr)
        self._update_available_views(available)
        should_decompile = True
        if available:
            chosen_flavor = flavor if flavor in available else available[0]
            cached = self.instance.kb.structured_code[(self._function.addr, chosen_flavor)].codegen
            if not isinstance(cached, DummyStructuredCodeGenerator):
                should_decompile = False
                self.codegen.am_obj = cached
                self.codegen.am_event()
                self._focus_core(focus, focus_addr)

        if should_decompile:
            self.decompile(focus=focus, focus_addr=focus_addr, flavor=flavor)
        self._last_function = self._function.am_obj

        console_view = self.workspace.view_manager.first_view_in_category("console")
        if console_view is not None:
            console_view.set_current_function(self._function.am_obj)

    def _on_cursor_position_changed(self):
        if self._doc is None:
            return

        cursor = self._textedit.textCursor()
        pos = cursor.position()
        selected_node = self._doc.get_node_at_position(pos)
        if selected_node is not None:
            # find all related text chunks and highlight them all
            chunks = self._doc.find_related_text_chunks(selected_node)
            # highlight these chunks
            self.highlight_chunks(chunks)
        else:
            self.highlight_chunks([])

        self.current_node.am_obj = selected_node
        self.current_node.am_event()

    def _on_mouse_doubleclicked(self):
        if self._doc is None:
            return

        cursor = self._textedit.textCursor()
        pos = cursor.position()
        selected_node = self._doc.get_node_at_position(pos)
        if selected_node is not None:
            if isinstance(selected_node, CFunctionCall):
                # decompile this new function
                if selected_node.callee_func is not None:
                    self.jump_history.record_address(selected_node.tags["ins_addr"])
                    self.jump_history.jump_to(selected_node.callee_func.addr)
                    self.workspace.decompile_function(selected_node.callee_func, view=self)
            elif isinstance(selected_node, CConstant):
                # jump to highlighted constants
                if selected_node.reference_values is not None and selected_node.value is not None:
                    self.workspace.jump_to(selected_node.value)
            elif isinstance(selected_node, CVariable):
                var = selected_node.variable
                if var and isinstance(var, SimMemoryVariable):
                    self.workspace.jump_to(var.addr)

    def jump_to(self, addr: int, src_ins_addr=None):  # pylint:disable=unused-argument
        self.addr.am_obj = addr
        self.addr.am_event()

    def jump_back(self):
        addr = self.jump_history.backtrack()
        if addr is None:
            self.close()
        else:
            self.jump_to(addr)

    def popup_jumpto_dialog(self):
        view = self.workspace._get_or_create_view("disassembly", DisassemblyView)
        JumpTo(view, parent=self).exec_()
        view.decompile_current_function()

    def jump_forward(self):
        addr = self.jump_history.forwardstep()
        if addr is not None:
            self.jump_to(addr)

    def jump_to_history_position(self, pos: int):
        addr = self.jump_history.step_position(pos)
        if addr is not None:
            self.jump_to(addr)

    def keyPressEvent(self, event):
        key = event.key()
        if key == Qt.Key_Tab:
            # Switch back to disassembly view
            self.workspace.jump_to(self.addr.am_obj)
            return True
        elif key == Qt.Key_G:
            # jump to window
            self.popup_jumpto_dialog()
            return True
        elif key == Qt.Key_Escape:
            self.jump_back()
            return True
        elif key == Qt.Key_Space and not self.codegen.am_none:
            flavor = self.codegen.flavor
            flavors = self.instance.kb.structured_code.available_flavors(self._function.addr)
            idx = flavors.index(flavor)
            newidx = (idx + 1) % len(flavors)
            self.codegen.am_obj = self.instance.kb.structured_code[(self._function.addr, flavors[newidx])].codegen
            self.codegen.am_event()
            return True

        return super().keyPressEvent(event)

    def _update_available_views(self, available):
        for _ in range(self._view_selector.count()):
            self._view_selector.removeItem(0)
        self._view_selector.addItems(available)
        self._view_selector.setVisible(len(available) >= 2)

    def _on_view_selector_changed(self, index):
        if not self._function.am_none:
            key = (self._function.addr, self._view_selector.itemText(index))
            self.codegen.am_obj = self.instance.kb.structured_code[key].codegen
            self.codegen.am_event()

    #
    # Private methods
    #

    def _init_widgets(self):
        window = QMainWindow()
        window.setWindowFlags(Qt.Widget)

        # pseudo code text box
        self._textedit = QCCodeEdit(self)
        self._textedit.setTextInteractionFlags(Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse)
        self._textedit.setLineWrapMode(QCCodeEdit.NoWrap)
        textedit_dock = QDockWidget("Code", self._textedit)
        window.setCentralWidget(textedit_dock)
        textedit_dock.setWidget(self._textedit)

        # decompilation
        self._options = QDecompilationOptions(self, self.instance)
        options_dock = QDockWidget("Decompilation Options", self._options)
        window.addDockWidget(Qt.RightDockWidgetArea, options_dock)
        options_dock.setWidget(self._options)

        # status bar
        status_bar = QFrame()
        self._nav_toolbar = NavToolbar(
            self.jump_history, self.jump_back, self.jump_forward, self.jump_to_history_position, True, self
        )
        self._view_selector = QComboBox()
        self._view_selector.addItems(["Pseudocode"])
        self._view_selector.activated.connect(self._on_view_selector_changed)
        status_layout = QHBoxLayout()
        status_layout.addWidget(self._nav_toolbar.qtoolbar())
        status_layout.addStretch(0)
        status_layout.addWidget(self._view_selector)
        status_layout.setContentsMargins(3, 3, 3, 3)
        status_layout.setSpacing(3)
        status_bar.setLayout(status_layout)

        inner_layout = QHBoxLayout()
        inner_layout.addWidget(window)
        inner_layout.setContentsMargins(0, 0, 0, 0)
        inner_layout.setSpacing(0)
        inner_widget = QWidget()
        inner_widget.setLayout(inner_layout)

        outer_layout = QVBoxLayout()
        outer_layout.addWidget(status_bar)
        outer_layout.addWidget(inner_widget)
        outer_layout.setContentsMargins(0, 0, 0, 0)
        outer_layout.setSpacing(0)

        self.setLayout(outer_layout)

        self._textedit.focusWidget()

        self.workspace.plugins.instrument_code_view(self)



import logging

from PySide6.QtCore import QSize, Qt
from PySide6.QtWidgets import QDialog, QVBoxLayout

from angrmanagement.logic import GlobalInfo
from angrmanagement.ui.widgets.qxref_viewer import QXRefViewer

_l = logging.getLogger(__name__)


class XRefDialog(QDialog):
    """
    Dialog displaying cross-references.
    """

    def __init__(
        self,
        addr=None,
        variable_manager=None,
        variable=None,
        xrefs_manager=None,
        dst_addr=None,
        instance=None,
        disassembly_view=None,
        parent=None,
    ):
        super().__init__(parent)

        self._variable_manager = variable_manager
        self._variable = variable
        self._xrefs_manager = xrefs_manager
        self._addr = addr  # current address
        self._dst_addr = dst_addr
        self._instance = instance
        self._disassembly_view = disassembly_view

        if variable is not None:
            self.setWindowTitle(f"XRefs to variable {variable.name}({variable.ident})")
        elif dst_addr is not None:
            # is there a label for it?
            try:
                lbl = self._instance.kb.labels.get(dst_addr)
            except KeyError:
                lbl = None
            if lbl is not None:
                self.setWindowTitle(f"XRefs to {lbl}")
            else:
                self.setWindowTitle(f"XRefs to address {dst_addr:#x}")
        else:
            raise ValueError("Either variable or dst_addr must be specified.")

        self._init_widgets()

        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint | Qt.WindowCloseButtonHint)
        self.setMinimumSize(self.sizeHint())
        self.adjustSize()

    def sizeHint(self, *args, **kwargs):  # pylint: disable=unused-argument,no-self-use
        return QSize(600, 400)

    def _init_widgets(self):
        # xref viewer
        xref_viewer = QXRefViewer(
            addr=self._addr,
            variable_manager=self._variable_manager,
            variable=self._variable,
            xrefs_manager=self._xrefs_manager,
            dst_addr=self._dst_addr,
            instance=self._instance,
            xref_dialog=self,
            parent=self,
        )

        layout = QVBoxLayout()
        layout.addWidget(xref_viewer)

        self.setLayout(layout)

    def jump_to(self, addr):
        self.close()
        disasm_view = self._disassembly_view
        if disasm_view is None:
            GlobalInfo.main_window.workspace.jump_to(addr)
        else:
            disasm_view.jump_to(addr, src_ins_addr=self._addr)
            disasm_view.focus()



import logging
from pathlib import Path
from typing import TYPE_CHECKING, List

from PySide6.QtCore import Qt
from PySide6.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QFileDialog,
    QFrame,
    QGroupBox,
    QListWidget,
    QListWidgetItem,
    QMessageBox,
    QPushButton,
    QVBoxLayout,
)

from angrmanagement.plugins import load_plugin_description

if TYPE_CHECKING:
    from angrmanagement.plugins import PluginDescription, PluginManager


_l = logging.getLogger(__name__)


class QPluginListWidgetItem(QListWidgetItem):
    """
    Plugin list item.
    """

    def __init__(self, plugin_desc, **kwargs):
        super().__init__(**kwargs)
        self.plugin_desc: PluginDescription = plugin_desc
        self.setText(plugin_desc.name)


# TODO: Add plugin settings, reloading, etc.


class LoadPlugins(QDialog):
    """
    Dialog to display loaded plugins, enable/disable plugins, and load new plugins.
    """

    def __init__(self, plugin_mgr, parent=None):
        super().__init__(parent)

        self._pm: PluginManager = plugin_mgr
        self._installed_plugin_list: QListWidget

        self.setWindowTitle("Installed Plugins")
        self.main_layout = QVBoxLayout()

        self._init_widgets()
        self._populate_installed_plugin_list()
        self.setLayout(self.main_layout)

    #
    # Private methods
    #

    def _init_plugin_list(self):
        plugin_group = QGroupBox("Plugins")
        self._installed_plugin_list = QListWidget(self)

        sublayout = QVBoxLayout()
        sublayout.addWidget(self._installed_plugin_list)
        plugin_group.setLayout(sublayout)

        layout = QVBoxLayout()
        layout.addWidget(plugin_group)

        frame = QFrame(self)
        frame.setLayout(layout)

        self.main_layout.addWidget(frame)

    def _populate_installed_plugin_list(self):
        for _, desc in self._pm.loaded_plugins.items():
            plugin_item = QPluginListWidgetItem(plugin_desc=desc)
            if self._pm.get_plugin_instance_by_name(desc.shortname) is not None:
                plugin_item.setCheckState(Qt.Checked)
            else:
                plugin_item.setCheckState(Qt.Unchecked)
            self._installed_plugin_list.addItem(plugin_item)

    def _init_widgets(self):
        load_button = QPushButton(self)
        load_button.setText("Load Plugin")
        load_button.clicked.connect(self._on_load_clicked)
        self.main_layout.addWidget(load_button)

        self._init_plugin_list()

        buttons = QDialogButtonBox(parent=self)
        buttons.setStandardButtons(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Ok)
        buttons.accepted.connect(self._on_ok_clicked)
        buttons.rejected.connect(self.close)
        self.main_layout.addWidget(buttons)

    #
    # Event handlers
    #

    def _on_ok_clicked(self):
        list_items: List[QPluginListWidgetItem] = self._installed_plugin_list.findItems("*", Qt.MatchWildcard)
        for i in list_items:
            checked = i.checkState() == Qt.Checked

            if checked and self._pm.get_plugin_instance_by_name(i.plugin_desc.shortname) is None:
                self._pm.activate_plugin_by_name(i.plugin_desc.shortname)
            elif not checked and self._pm.get_plugin_instance_by_name(i.plugin_desc.shortname) is not None:
                self._pm.deactivate_plugin_by_name(i.plugin_desc.shortname)

        self._pm.save_enabled_plugins_to_config()
        self.close()

    def _on_load_clicked(self):
        file_path, _ = QFileDialog.getOpenFileName(
            self, "Open a plugin description file (plugin.toml)", "", "Toml files (*.toml)"
        )
        if not file_path:
            return
        plugins = load_plugin_description(str(Path(file_path).parent))

        if not plugins:
            QMessageBox.warning(self, "Error", "File contained no plugin descriptions")
            return

        for plugin in plugins:
            plugin_item = QPluginListWidgetItem(plugin_desc=plugin)
            plugin_item.setCheckState(Qt.Unchecked)
            self._pm.loaded_plugins[plugin.shortname] = plugin
            self._installed_plugin_list.addItem(plugin_item)



from typing import TYPE_CHECKING, Callable, Optional, Type

if TYPE_CHECKING:
    from angrmanagement.ui.views import BaseView
    from angrmanagement.ui.workspace import Workspace


class Command:
    """
    Command to be run.
    """

    _name: Optional[str] = None
    _caption: Optional[str] = None

    @property
    def name(self) -> str:
        """
        Short name for invocation. By default this name will be derived from the class name.
        """
        return self._name or self.__class__.__name__

    @property
    def caption(self) -> str:
        """
        Message to be displayed to identify the command, e.g. in the command palette.
        """
        return self._caption or self.name

    @property
    def is_visible(self) -> bool:
        """
        Determines whether this command should be displayed or not.
        """
        return True

    def run(self) -> None:
        """
        Runs the command.
        """


class BasicCommand(Command):
    """
    Basic command to invoke a callable.
    """

    def __init__(self, name: str, caption: str, action: Callable):
        self._name = name
        self._caption = caption
        self._action: Callable = action

    def run(self):
        self._action()


class ViewCommand(Command):
    """
    Commands to invoke a callable on a view.
    """

    def __init__(self, name: str, caption: str, action: Callable, view_class: Type["BaseView"], workspace: "Workspace"):
        self._name = name
        self._caption = caption
        self._action: Callable = action
        self._view_class: Type[BaseView] = view_class
        self._workspace: Workspace = workspace

    @property
    def is_visible(self) -> bool:
        view = self._workspace.view_manager.most_recently_focused_view
        return isinstance(view, self._view_class)

    def run(self) -> None:
        view = self._workspace.view_manager.most_recently_focused_view
        if isinstance(view, self._view_class):
            self._action(view)



import difflib
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple

from PySide6.QtCore import QAbstractItemModel, QMargins, QModelIndex, QRectF, QSize, Qt
from PySide6.QtGui import QBrush, QColor, QTextDocument
from PySide6.QtWidgets import QDialog, QLineEdit, QListView, QStyle, QStyledItemDelegate, QVBoxLayout
from thefuzz import process

from angrmanagement.config import Conf

if TYPE_CHECKING:
    from angr.knowledge_plugins.functions import Function

    from angrmanagement.logic.commands import Command
    from angrmanagement.ui.workspace import Workspace


class PaletteModel(QAbstractItemModel):
    """
    Data provider for item palette.
    """

    def __init__(self, workspace: "Workspace"):
        super().__init__()
        self.workspace: Workspace = workspace
        self._available_items: List[Any] = self.get_items()
        self._item_to_caption: Dict[Any, str] = {
            item: self.get_caption_for_item(item) for item in self._available_items
        }
        self._filtered_items: List[Any] = self._available_items
        self._filter_text: str = ""

    def rowCount(self, _):
        return len(self._filtered_items)

    def columnCount(self, _):  # pylint:disable=no-self-use
        return 1

    def index(self, row, col, _):
        return self.createIndex(row, col, None)

    def parent(self, _):  # pylint:disable=no-self-use
        return QModelIndex()

    def data(self, index, role):
        if not index.isValid() or role != Qt.DisplayRole:
            return None
        row = index.row()
        return self._filtered_items[row] if row < len(self._filtered_items) else None

    def set_filter_text(self, query: str) -> None:
        """
        Filter the list of available items by captions matching `query`.
        """
        self.beginResetModel()
        self._filter_text = query
        if query == "":
            self._filtered_items = self._available_items
        else:
            self._filtered_items = [item for _, _, item in process.extract(query, self._item_to_caption, limit=50)]
        self.endResetModel()

    def get_items(self) -> List[Any]:  # pylint:disable=no-self-use
        return []

    def get_caption_for_item(self, item: Any) -> str:  # pylint:disable=no-self-use,unused-argument
        return ""

    def get_subcaption_for_item(self, item: Any) -> Optional[str]:  # pylint:disable=no-self-use,unused-argument
        return None

    def get_annotation_for_item(self, item: Any) -> Optional[str]:  # pylint:disable=no-self-use,unused-argument
        return None

    # pylint:disable=no-self-use,unused-argument
    def get_icon_color_and_text_for_item(self, item: Any) -> Tuple[Optional[QColor], str]:
        return (None, "")


class CommandPaletteModel(PaletteModel):
    """
    Data provider for command palette.
    """

    def get_items(self) -> List["Command"]:
        return sorted(
            [cmd for cmd in self.workspace.command_manager.get_commands() if cmd.is_visible],
            key=lambda cmd: cmd.caption,
        )

    def get_caption_for_item(self, item: "Command") -> str:
        return item.caption


class GotoPaletteModel(PaletteModel):
    """
    Data provider for goto palette.
    """

    def get_items(self) -> List["Function"]:
        items = []

        instance = self.workspace.main_instance
        if instance and not instance.project.am_none:
            project = instance.project.am_obj
            items.extend([func for _, func in project.kb.functions.items()])

        return items

    def get_icon_color_and_text_for_item(self, item: "Function") -> Tuple[Optional[QColor], str]:
        if item.is_syscall:
            color = Conf.function_table_syscall_color
        elif item.is_plt:
            color = Conf.function_table_plt_color
        elif item.is_simprocedure:
            color = Conf.function_table_simprocedure_color
        elif item.alignment:
            color = Conf.function_table_alignment_color
        else:
            color = Qt.gray
        return (color, "f")

    def get_caption_for_item(self, item: "Function") -> str:
        return item.name

    def get_annotation_for_item(self, item: "Function") -> str:
        return f"{item.addr:x}"


class PaletteItemDelegate(QStyledItemDelegate):
    """
    Delegate to draw individual palette entries.

    Query sub-sequence matches against item captions are shown in bold.
    """

    icon_width = 25

    def __init__(self, display_icons: bool = True):
        super().__init__()
        self._display_icons = display_icons

    @staticmethod
    def _get_text_document(index):
        model: PaletteModel = index.model()
        item = index.data()
        text_in = model.get_caption_for_item(item)

        if not model._filter_text:
            text_out = text_in
        else:
            # Render matching sub-sequences in bold
            matcher = difflib.SequenceMatcher(None, text_in.upper(), model._filter_text.upper())
            text_out = ""
            last_idx = 0
            for idx, _, size in matcher.get_matching_blocks():
                text_out += text_in[last_idx:idx] + f"<b>{text_in[idx:idx + size]}</b>"
                last_idx = idx + size
            text_out += text_in[last_idx:]

        subcaption = model.get_subcaption_for_item(item)
        if subcaption:
            text_out += "<br/><sub>" + subcaption + "</sub>"

        td = QTextDocument()
        td.setHtml(text_out)
        return td

    def paint(self, painter, option, index):
        if index.column() == 0:
            if option.state & QStyle.State_Selected:
                b = QBrush(option.palette.highlight())
                painter.fillRect(option.rect, b)

            model: PaletteModel = index.model()
            item = index.data()

            td = self._get_text_document(index)
            td.setDefaultFont(option.font)

            annotation_text = model.get_annotation_for_item(item)
            if annotation_text:
                painter.drawText(
                    option.rect.marginsRemoved(QMargins(3, 3, 3, 3)), Qt.AlignRight | Qt.AlignVCenter, annotation_text
                )

            painter.save()
            painter.translate(option.rect.topLeft())

            if self._display_icons:
                icon_rect = QRectF(0, 0, self.icon_width, td.size().height())
                icon_color, icon_text = model.get_icon_color_and_text_for_item(item)
                if icon_color:
                    painter.fillRect(icon_rect, QBrush(QColor(icon_color)))
                if icon_text:
                    painter.setPen(Qt.white)
                    painter.drawText(icon_rect, Qt.AlignCenter, icon_text)
                painter.translate(self.icon_width, 0)

            td.drawContents(painter)
            painter.restore()
        else:
            super().paint(painter, option, index)

    def sizeHint(self, option, index) -> QSize:
        if index.column() == 0:
            td = self._get_text_document(index)
            td.setDefaultFont(option.font)
            s = td.size()
            width = s.width()
            if self._display_icons:
                width += self.icon_width
            return QSize(width, s.height())
        return super().sizeHint(option, index)


class PaletteDialog(QDialog):
    """
    Dialog for selecting an item from a palette.
    """

    def __init__(self, model, delegate=None, parent=None):
        super().__init__(parent)
        self._model = model
        self._delegate = delegate or PaletteItemDelegate()
        self._init_widgets()

        self.selected_item = None

        self.setWindowTitle("Palette")
        self.setMinimumSize(self.sizeHint())
        self.adjustSize()

    def sizeHint(self):  # pylint:disable=no-self-use
        return QSize(500, 400)

    #
    # Private methods
    #

    def _init_widgets(self):
        self._layout: QVBoxLayout = QVBoxLayout()

        self._query: QLineEdit = QLineEdit(self)
        self._query.textChanged.connect(self._set_filter_text)
        self._layout.addWidget(self._query)

        self._view: QListView = QListView(self)
        self._view.setModel(self._model)
        self._view.setItemDelegate(self._delegate)
        self._view.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self._view.clicked.connect(self.accept)
        self._layout.addWidget(self._view)

        self.setLayout(self._layout)
        self._set_filter_text("")

    def _set_filter_text(self, text):
        self._model.set_filter_text(text)
        self._view.setCurrentIndex(self._model.index(0, 0, None))

    def _get_selected(self):
        for i in self._view.selectedIndexes():
            return i.data()
        return None

    #
    # Event handlers
    #

    def keyPressEvent(self, event):
        key = event.key()
        if key in {Qt.Key_Up, Qt.Key_Down}:
            self._view.keyPressEvent(event)
        elif key in {Qt.Key_Enter, Qt.Key_Return}:
            self.accept()
        else:
            super().keyPressEvent(event)

    def accept(self):
        self.selected_item = self._get_selected()
        super().accept()


class CommandPaletteDialog(PaletteDialog):
    """
    Dialog for selecting commands.
    """

    def __init__(self, workspace, parent=None):
        super().__init__(CommandPaletteModel(workspace), PaletteItemDelegate(display_icons=False), parent)
        self.setWindowTitle("Command Palette")


class GotoPaletteDialog(PaletteDialog):
    """
    Dialog for selecting navigation targets.
    """

    def __init__(self, workspace, parent=None):
        super().__init__(GotoPaletteModel(workspace), parent=parent)
        self.setWindowTitle("Goto Anything")



import logging
import struct

from angrmanagement.plugins import BasePlugin

from .search_view import SearchView

logger = logging.getLogger(__name__)


class ValueSearch(BasePlugin):
    def __init__(self, workspace):
        super().__init__(workspace)

        self._endness_encoding = None
        self._create_search_view()

    def _find_endness_encoding(self):
        endness = self.workspace.main_instance.project.arch.memory_endness
        self._endness_encoding = ">" if endness.endswith("BE") else "<"

    #
    # UI Callback Handlers
    #

    def handle_click_menu(self, idx):
        pass

    def color_insn(self, addr, selected, disasm_view):
        pass

    def teardown(self):
        self._destroy_search_view()

    def _create_search_view(self):
        self.search_view = SearchView(self, self.workspace, self.workspace.main_instance, "center")
        self.workspace.add_view(self.search_view)

    def _destroy_search_view(self):
        self.workspace.remove_view(self.search_view)

    #
    # helpers
    #

    def search_by_bytes(self, value: bytes, alignment: int):
        if value is None:
            return []

        addrs = self.workspace.main_instance.project.loader.memory.find(value)
        if alignment <= 1:
            return list(addrs)
        return [addr for addr in addrs if addr % alignment == 0]

    def _float_to_bytes(self, f_value: float):
        if self._endness_encoding is None:
            self._find_endness_encoding()

        try:
            enc_value = struct.pack(f"{self._endness_encoding}f", f_value)
        except struct.error:
            enc_value = None

        return enc_value

    def _int_to_bytes(self, i_value: int):
        if self._endness_encoding is None:
            self._find_endness_encoding()

        try:
            enc_value = struct.pack(f"{self._endness_encoding}I", i_value)
        except struct.error:
            enc_value = None

        return enc_value

    #
    # callbacks
    #

    def on_search_trigger(self, value: str, type_: str, alignment: int):
        if type_ == "int":
            i_val = int(value, 0)
            value = self._int_to_bytes(i_val)
        elif type_ == "float":
            f_val = float(value)
            value = self._float_to_bytes(f_val)
        else:
            value = value.encode().decode("unicode_escape").encode("latin-1")

        return self.search_by_bytes(value, alignment), value
